@@ -22,11 +22,12 @@
 #include <string.h> // memcpy
 #include <stdlib.h> // malloc, free
 
+#include <libdivecomputer/oceanic_atom2.h>
+
+#include "context-private.h"
 #include "device-private.h"
 #include "oceanic_common.h"
-#include "oceanic_atom2.h"
 #include "serial.h"
-#include "utils.h"
 #include "array.h"
 #include "ringbuffer.h"
 #include "checksum.h"
@@ -35,7 +36,7 @@
 
 #define EXITCODE(rc) \
 ( \
-	rc == -1 ? DEVICE_STATUS_IO : DEVICE_STATUS_TIMEOUT \
+	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
 )
 
 #define ACK 0x5A
@@ -44,18 +45,15 @@
 typedef struct oceanic_atom2_device_t {
 	oceanic_common_device_t base;
 	serial_t *port;
-	unsigned char version[PAGESIZE];
 } oceanic_atom2_device_t;
 
-static device_status_t oceanic_atom2_device_version (device_t *abstract, unsigned char data[], unsigned int size);
-static device_status_t oceanic_atom2_device_read (device_t *abstract, unsigned int address, unsigned char data[], unsigned int size);
-static device_status_t oceanic_atom2_device_write (device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size);
-static device_status_t oceanic_atom2_device_close (device_t *abstract);
+static dc_status_t oceanic_atom2_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size);
+static dc_status_t oceanic_atom2_device_write (dc_device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size);
+static dc_status_t oceanic_atom2_device_close (dc_device_t *abstract);
 
 static const device_backend_t oceanic_atom2_device_backend = {
-	DEVICE_TYPE_OCEANIC_ATOM2,
+	DC_FAMILY_OCEANIC_ATOM2,
 	oceanic_common_device_set_fingerprint, /* set_fingerprint */
-	oceanic_atom2_device_version, /* version */
 	oceanic_atom2_device_read, /* read */
 	oceanic_atom2_device_write, /* write */
 	oceanic_common_device_dump, /* dump */
@@ -63,28 +61,66 @@ static const device_backend_t oceanic_atom2_device_backend = {
 	oceanic_atom2_device_close /* close */
 };
 
-static const unsigned char aeris_atmosai_version[] = "ATMOSAI R\0\0 512K";
-static const unsigned char aeris_epic_version[]  = "2M EPIC r\0\0 512K";
-static const unsigned char aeris_f10_version[] = "FREEWAER \0\0 512K";
-static const unsigned char oceanic_proplus2_version[] = "PROPLUS2 \0\0 512K";
-static const unsigned char oceanic_atom1_version[] = "ATOM rev\0\0  256K";
-static const unsigned char oceanic_atom2_version[] = "2M ATOM r\0\0 512K";
-static const unsigned char oceanic_atom3_version[] = "OCEATOM3 \0\0 1024";
-static const unsigned char oceanic_atom31_version[] = "ATOM31  \0\0  1024";
-static const unsigned char oceanic_vt4_version[]   = "OCEANVT4 \0\0 1024";
-static const unsigned char oceanic_vt41_version[]  = "OCEAVT41 \0\0 1024";
-static const unsigned char oceanic_geo2_version[]  = "OCEGEO20 \0\0 512K";
-static const unsigned char oceanic_oc1a_version[]  = "OCWATCH R\0\0 1024";
-static const unsigned char oceanic_oc1b_version[]  = "OC1WATCH \0\0 1024";
-static const unsigned char oceanic_veo1_version[]  = "OCEVEO10 \0\0   8K";
-static const unsigned char oceanic_veo2_version[]  = "OCEVEO20 \0\0 512K";
-static const unsigned char oceanic_veo3_version[]  = "OCEVEO30 \0\0 512K";
-static const unsigned char sherwood_insight_version[] = "INSIGHT2 \0\0 512K";
-static const unsigned char sherwood_wisdom2_version[] = "WISDOM R\0\0  512K";
-static const unsigned char tusa_element2_version[] = "ELEMENT2 \0\0 512K";
-static const unsigned char tusa_zen_version[]      = "TUSAZEN \0\0  512K";
-static const unsigned char tusa_zenair_version[]   = "TUZENAIR \0\0 512K";
+static const oceanic_common_version_t aeris_f10_version[] = {
+	{"FREEWAER \0\0 512K"},
+};
 
+static const oceanic_common_version_t oceanic_atom1_version[] = {
+	{"ATOM rev\0\0  256K"},
+};
+
+static const oceanic_common_version_t oceanic_atom2_version[] = {
+	{"2M ATOM r\0\0 512K"},
+};
+
+static const oceanic_common_version_t oceanic_atom2a_version[] = {
+	{"MANTA  R\0\0  512K"},
+	{"WISDOM R\0\0  512K"},
+	{"INSIGHT2 \0\0 512K"},
+	{"OCEVEO30 \0\0 512K"},
+	{"ATMOSAI R\0\0 512K"},
+	{"PROPLUS2 \0\0 512K"},
+	{"OCEGEO20 \0\0 512K"},
+};
+
+static const oceanic_common_version_t oceanic_atom2b_version[] = {
+	{"ELEMENT2 \0\0 512K"},
+	{"OCEVEO20 \0\0 512K"},
+	{"TUSAZEN \0\0  512K"},
+};
+
+static const oceanic_common_version_t oceanic_atom2c_version[] = {
+	{"2M EPIC r\0\0 512K"},
+	{"EPIC1  R\0\0  512K"},
+};
+
+static const oceanic_common_version_t oceanic_default_version[] = {
+	{"OCE VT3 R\0\0 512K"},
+	{"ELITET3 R\0\0 512K"},
+	{"ELITET31 \0\0 512K"},
+	{"DATAMASK \0\0 512K"},
+	{"COMPMASK \0\0 512K"},
+	{"HOLLDG03 \0\0 512K"},
+};
+
+static const oceanic_common_version_t tusa_zenair_version[] = {
+	{"TUZENAIR \0\0 512K"},
+};
+
+static const oceanic_common_version_t oceanic_oc1_version[] = {
+	{"OCWATCH R\0\0 1024"},
+	{"OCEANVT4 \0\0 1024"},
+	{"OC1WATCH \0\0 1024"},
+	{"OCEATOM3 \0\0 1024"},
+	{"OCSWATCH \0\0 1024"},
+	{"OCEAVT41 \0\0 1024"},
+	{"ATOM31  \0\0  1024"},
+	{"AERISAIR \0\0 1024"},
+};
+
+static const oceanic_common_version_t oceanic_veo1_version[] = {
+	{"OCEVEO10 \0\0   8K"},
+};
 
 static const oceanic_common_layout_t aeris_f10_layout = {
 	0x10000, /* memsize */
@@ -205,7 +241,7 @@ static const oceanic_common_layout_t oceanic_veo1_layout = {
 
 
 static int
-device_is_oceanic_atom2 (device_t *abstract)
+device_is_oceanic_atom2 (dc_device_t *abstract)
 {
 	if (abstract == NULL)
 		return 0;
@@ -214,18 +250,18 @@ device_is_oceanic_atom2 (device_t *abstract)
 }
 
 
-static device_status_t
+static dc_status_t
 oceanic_atom2_send (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char ack)
 {
-	device_t *abstract = (device_t *) device;
+	dc_device_t *abstract = (dc_device_t *) device;
 
 	if (device_is_cancelled (abstract))
-		return DEVICE_STATUS_CANCELLED;
+		return DC_STATUS_CANCELLED;
 
 	// Send the command to the dive computer.
 	int n = serial_write (device->port, command, csize);
 	if (n != csize) {
-		WARNING ("Failed to send the command.");
+		ERROR (abstract->context, "Failed to send the command.");
 		return EXITCODE (n);
 	}
 
@@ -233,23 +269,25 @@ oceanic_atom2_send (oceanic_atom2_device_t *device, const unsigned char command[
 	unsigned char response = 0;
 	n = serial_read (device->port, &response, 1);
 	if (n != 1) {
-		WARNING ("Failed to receive the answer.");
+		ERROR (abstract->context, "Failed to receive the answer.");
 		return EXITCODE (n);
 	}
 
 	// Verify the response of the dive computer.
 	if (response != ack) {
-		WARNING ("Unexpected answer start byte(s).");
-		return DEVICE_STATUS_PROTOCOL;
+		ERROR (abstract->context, "Unexpected answer start byte(s).");
+		return DC_STATUS_PROTOCOL;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize)
 {
+	dc_device_t *abstract = (dc_device_t *) device;
+
 	// Send the command to the device. If the device responds with an
 	// ACK byte, the command was received successfully and the answer
 	// (if any) follows after the ACK byte. If the device responds with
@@ -257,9 +295,9 @@ oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char comm
 	// returning an error.
 
 	unsigned int nretries = 0;
-	device_status_t rc = DEVICE_STATUS_SUCCESS;
-	while ((rc = oceanic_atom2_send (device, command, csize, ACK)) != DEVICE_STATUS_SUCCESS) {
-		if (rc != DEVICE_STATUS_TIMEOUT && rc != DEVICE_STATUS_PROTOCOL)
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	while ((rc = oceanic_atom2_send (device, command, csize, ACK)) != DC_STATUS_SUCCESS) {
+		if (rc != DC_STATUS_TIMEOUT && rc != DC_STATUS_PROTOCOL)
 			return rc;
 
 		// Abort if the maximum number of retries is reached.
@@ -267,7 +305,7 @@ oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char comm
 			return rc;
 
 		// Delay the next attempt.
-		serial_sleep (100);
+		serial_sleep (device->port, 100);
 		serial_flush (device->port, SERIAL_QUEUE_INPUT);
 	}
 
@@ -275,7 +313,7 @@ oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char comm
 		// Receive the answer of the dive computer.
 		int n = serial_read (device->port, answer, asize);
 		if (n != asize) {
-			WARNING ("Failed to receive the answer.");
+			ERROR (abstract->context, "Failed to receive the answer.");
 			return EXITCODE (n);
 		}
 
@@ -283,75 +321,74 @@ oceanic_atom2_transfer (oceanic_atom2_device_t *device, const unsigned char comm
 		unsigned char crc = answer[asize - 1];
 		unsigned char ccrc = checksum_add_uint8 (answer, asize - 1, 0x00);
 		if (crc != ccrc) {
-			WARNING ("Unexpected answer CRC.");
-			return DEVICE_STATUS_PROTOCOL;
+			ERROR (abstract->context, "Unexpected answer checksum.");
+			return DC_STATUS_PROTOCOL;
 		}
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 oceanic_atom2_quit (oceanic_atom2_device_t *device)
 {
 	// Send the command to the dive computer.
 	unsigned char command[4] = {0x6A, 0x05, 0xA5, 0x00};
-	device_status_t rc = oceanic_atom2_send (device, command, sizeof (command), NAK);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = oceanic_atom2_send (device, command, sizeof (command), NAK);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-oceanic_atom2_device_open (device_t **out, const char* name)
+dc_status_t
+oceanic_atom2_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
 	if (out == NULL)
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t *) malloc (sizeof (oceanic_atom2_device_t));
 	if (device == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	oceanic_common_device_init (&device->base, &oceanic_atom2_device_backend);
+	oceanic_common_device_init (&device->base, context, &oceanic_atom2_device_backend);
 
 	// Set the default values.
 	device->port = NULL;
-	memset (device->version, 0, sizeof (device->version));
 
 	// Open the device.
-	int rc = serial_open (&device->port, name);
+	int rc = serial_open (&device->port, context, name);
 	if (rc == -1) {
-		WARNING ("Failed to open the serial port.");
+		ERROR (context, "Failed to open the serial port.");
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Set the serial communication protocol (38400 8N1).
 	rc = serial_configure (device->port, 38400, 8, SERIAL_PARITY_NONE, 1, SERIAL_FLOWCONTROL_NONE);
 	if (rc == -1) {
-		WARNING ("Failed to set the terminal attributes.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Set the timeout for receiving data (3000 ms).
 	if (serial_set_timeout (device->port, 3000) == -1) {
-		WARNING ("Failed to set the timeout.");
+		ERROR (context, "Failed to set the timeout.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Give the interface 100 ms to settle and draw power up.
-	serial_sleep (100);
+	serial_sleep (device->port, 100);
 
 	// Make sure everything is in a sane state.
 	serial_flush (device->port, SERIAL_QUEUE_BOTH);
@@ -359,61 +396,53 @@ oceanic_atom2_device_open (device_t **out, const char* name)
 	// Switch the device from surface mode into download mode. Before sending
 	// this command, the device needs to be in PC mode (automatically activated
 	// by connecting the device), or already in download mode.
-	device_status_t status = oceanic_atom2_device_version ((device_t *) device, device->version, sizeof (device->version));
-	if (status != DEVICE_STATUS_SUCCESS) {
+	dc_status_t status = oceanic_atom2_device_version ((dc_device_t *) device, device->base.version, sizeof (device->base.version));
+	if (status != DC_STATUS_SUCCESS) {
 		serial_close (device->port);
 		free (device);
 		return status;
 	}
 
 	// Override the base class values.
-	if (oceanic_common_match (oceanic_oc1a_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_oc1b_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_atom3_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_atom31_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_vt4_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_vt41_version, device->version, sizeof (device->version)))
-		device->base.layout = &oceanic_oc1_layout;
-	else if (oceanic_common_match (aeris_f10_version, device->version, sizeof (device->version)))
+	if (OCEANIC_COMMON_MATCH (device->base.version, aeris_f10_version)) {
 		device->base.layout = &aeris_f10_layout;
-	else if (oceanic_common_match (tusa_zenair_version, device->version, sizeof (device->version)))
-		device->base.layout = &tusa_zenair_layout;
-	else if (oceanic_common_match (oceanic_atom1_version, device->version, sizeof (device->version)))
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom1_version)) {
 		device->base.layout = &oceanic_atom1_layout;
-	else if (oceanic_common_match (sherwood_insight_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (sherwood_wisdom2_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (aeris_atmosai_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_geo2_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_veo3_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_proplus2_version, device->version, sizeof (device->version)) ||
-		(oceanic_common_match (oceanic_atom2_version, device->version, sizeof (device->version)) &&
-		 array_uint16_be (device->version + 0x09) >= 0x3349))
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom2_version)) {
+		if (array_uint16_be (device->base.version + 0x09) >= 0x3349) {
+			device->base.layout = &oceanic_atom2a_layout;
+		} else {
+			device->base.layout = &oceanic_atom2c_layout;
+		}
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom2a_version)) {
 		device->base.layout = &oceanic_atom2a_layout;
-	else if (oceanic_common_match (oceanic_veo2_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (tusa_element2_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (tusa_zen_version, device->version, sizeof (device->version)))
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom2b_version)) {
 		device->base.layout = &oceanic_atom2b_layout;
-	else if (oceanic_common_match (aeris_epic_version, device->version, sizeof (device->version)) ||
-		oceanic_common_match (oceanic_atom2_version, device->version, sizeof (device->version)))
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_atom2c_version)) {
 		device->base.layout = &oceanic_atom2c_layout;
-	else if (oceanic_common_match (oceanic_veo1_version, device->version, sizeof (device->version)))
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, tusa_zenair_version)) {
+		device->base.layout = &tusa_zenair_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_oc1_version)) {
+		device->base.layout = &oceanic_oc1_layout;
+	} else if (OCEANIC_COMMON_MATCH (device->base.version, oceanic_veo1_version)) {
 		device->base.layout = &oceanic_veo1_layout;
-	else
+	} else {
 		device->base.layout = &oceanic_default_layout;
+	}
 
-	*out = (device_t*) device;
+	*out = (dc_device_t*) device;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-oceanic_atom2_device_close (device_t *abstract)
+static dc_status_t
+oceanic_atom2_device_close (dc_device_t *abstract)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
 	if (! device_is_oceanic_atom2 (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Send the quit command.
 	oceanic_atom2_quit (device);
@@ -421,71 +450,68 @@ oceanic_atom2_device_close (device_t *abstract)
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Free memory.	
 	free (device);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-oceanic_atom2_device_keepalive (device_t *abstract)
+dc_status_t
+oceanic_atom2_device_keepalive (dc_device_t *abstract)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
 	if (! device_is_oceanic_atom2 (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Send the command to the dive computer.
 	unsigned char command[4] = {0x91, 0x05, 0xA5, 0x00};
-	device_status_t rc = oceanic_atom2_transfer (device, command, sizeof (command), NULL, 0);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = oceanic_atom2_transfer (device, command, sizeof (command), NULL, 0);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-oceanic_atom2_device_version (device_t *abstract, unsigned char data[], unsigned int size)
+dc_status_t
+oceanic_atom2_device_version (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
 	if (! device_is_oceanic_atom2 (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	if (size < PAGESIZE)
-		return DEVICE_STATUS_MEMORY;
+		return DC_STATUS_INVALIDARGS;
 
 	unsigned char answer[PAGESIZE + 1] = {0};
 	unsigned char command[2] = {0x84, 0x00};
-	device_status_t rc = oceanic_atom2_transfer (device, command, sizeof (command), answer, sizeof (answer));
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = oceanic_atom2_transfer (device, command, sizeof (command), answer, sizeof (answer));
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	memcpy (data, answer, PAGESIZE);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-oceanic_atom2_device_read (device_t *abstract, unsigned int address, unsigned char data[], unsigned int size)
+static dc_status_t
+oceanic_atom2_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
 	if (! device_is_oceanic_atom2 (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	if ((address % PAGESIZE != 0) ||
 		(size    % PAGESIZE != 0))
-		return DEVICE_STATUS_ERROR;
-	
-	// The data transmission is split in packages
-	// of maximum $PAGESIZE bytes.
+		return DC_STATUS_INVALIDARGS;
 
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
@@ -496,8 +522,8 @@ oceanic_atom2_device_read (device_t *abstract, unsigned int address, unsigned ch
 				(number >> 8) & 0xFF, // high
 				(number     ) & 0xFF, // low
 				0};
-		device_status_t rc = oceanic_atom2_transfer (device, command, sizeof (command), answer, sizeof (answer));
-		if (rc != DEVICE_STATUS_SUCCESS)
+		dc_status_t rc = oceanic_atom2_transfer (device, command, sizeof (command), answer, sizeof (answer));
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		memcpy (data, answer, PAGESIZE);
@@ -507,24 +533,21 @@ oceanic_atom2_device_read (device_t *abstract, unsigned int address, unsigned ch
 		data += PAGESIZE;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-oceanic_atom2_device_write (device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size)
+static dc_status_t
+oceanic_atom2_device_write (dc_device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size)
 {
 	oceanic_atom2_device_t *device = (oceanic_atom2_device_t*) abstract;
 
 	if (! device_is_oceanic_atom2 (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	if ((address % PAGESIZE != 0) ||
 		(size    % PAGESIZE != 0))
-		return DEVICE_STATUS_ERROR;
-
-	// The data transmission is split in packages
-	// of maximum $PAGESIZE bytes.
+		return DC_STATUS_INVALIDARGS;
 
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
@@ -534,8 +557,8 @@ oceanic_atom2_device_write (device_t *abstract, unsigned int address, const unsi
 				(number >> 8) & 0xFF, // high
 				(number     ) & 0xFF, // low
 				0x00};
-		device_status_t rc = oceanic_atom2_transfer (device, prepare, sizeof (prepare), NULL, 0);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		dc_status_t rc = oceanic_atom2_transfer (device, prepare, sizeof (prepare), NULL, 0);
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		// Write the package.
@@ -543,7 +566,7 @@ oceanic_atom2_device_write (device_t *abstract, unsigned int address, const unsi
 		memcpy (command, data, PAGESIZE);
 		command[PAGESIZE] = checksum_add_uint8 (command, PAGESIZE, 0x00);
 		rc = oceanic_atom2_transfer (device, command, sizeof (command), NULL, 0);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		nbytes += PAGESIZE;
@@ -551,5 +574,5 @@ oceanic_atom2_device_write (device_t *abstract, unsigned int address, const unsi
 		data += PAGESIZE;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
