@@ -21,15 +21,16 @@
 
 #include <stdlib.h>
 
-#include "oceanic_atom2.h"
+#include <libdivecomputer/oceanic_atom2.h>
+#include <libdivecomputer/units.h>
+
 #include "oceanic_common.h"
+#include "context-private.h"
 #include "parser-private.h"
 #include "array.h"
-#include "units.h"
-#include "utils.h"
 
 #define ATOM1       0x4250
-#define EPIC        0x4257
+#define EPICA       0x4257
 #define VT3         0x4258
 #define T3          0x4259
 #define ATOM2       0x4342
@@ -46,13 +47,17 @@
 #define VT4         0x4447
 #define OC1B        0x4449
 #define ATOM3       0x444C
+#define DG03        0x444D
+#define OCS         0x4450
 #define VT41        0x4452
+#define EPICB       0x4453
 #define ATOM31      0x4456
+#define A300AI      0x4457
 
 typedef struct oceanic_atom2_parser_t oceanic_atom2_parser_t;
 
 struct oceanic_atom2_parser_t {
-	parser_t base;
+	dc_parser_t base;
 	unsigned int model;
 	// Cached fields.
 	unsigned int cached;
@@ -60,14 +65,14 @@ struct oceanic_atom2_parser_t {
 	double maxdepth;
 };
 
-static parser_status_t oceanic_atom2_parser_set_data (parser_t *abstract, const unsigned char *data, unsigned int size);
-static parser_status_t oceanic_atom2_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime);
-static parser_status_t oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, unsigned int flags, void *value);
-static parser_status_t oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t callback, void *userdata);
-static parser_status_t oceanic_atom2_parser_destroy (parser_t *abstract);
+static dc_status_t oceanic_atom2_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
+static dc_status_t oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
+static dc_status_t oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
+static dc_status_t oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
+static dc_status_t oceanic_atom2_parser_destroy (dc_parser_t *abstract);
 
 static const parser_backend_t oceanic_atom2_parser_backend = {
-	PARSER_TYPE_OCEANIC_ATOM2,
+	DC_FAMILY_OCEANIC_ATOM2,
 	oceanic_atom2_parser_set_data, /* set_data */
 	oceanic_atom2_parser_get_datetime, /* datetime */
 	oceanic_atom2_parser_get_field, /* fields */
@@ -77,7 +82,7 @@ static const parser_backend_t oceanic_atom2_parser_backend = {
 
 
 static int
-parser_is_oceanic_atom2 (parser_t *abstract)
+parser_is_oceanic_atom2 (dc_parser_t *abstract)
 {
 	if (abstract == NULL)
 		return 0;
@@ -86,21 +91,21 @@ parser_is_oceanic_atom2 (parser_t *abstract)
 }
 
 
-parser_status_t
-oceanic_atom2_parser_create (parser_t **out, unsigned int model)
+dc_status_t
+oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int model)
 {
 	if (out == NULL)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) malloc (sizeof (oceanic_atom2_parser_t));
 	if (parser == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return PARSER_STATUS_MEMORY;
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, &oceanic_atom2_parser_backend);
+	parser_init (&parser->base, context, &oceanic_atom2_parser_backend);
 
 	// Set the default values.
 	parser->model = model;
@@ -108,44 +113,44 @@ oceanic_atom2_parser_create (parser_t **out, unsigned int model)
 	parser->divetime = 0;
 	parser->maxdepth = 0.0;
 
-	*out = (parser_t*) parser;
+	*out = (dc_parser_t*) parser;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-oceanic_atom2_parser_destroy (parser_t *abstract)
+static dc_status_t
+oceanic_atom2_parser_destroy (dc_parser_t *abstract)
 {
 	if (! parser_is_oceanic_atom2 (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Free memory.
 	free (abstract);
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-oceanic_atom2_parser_set_data (parser_t *abstract, const unsigned char *data, unsigned int size)
+static dc_status_t
+oceanic_atom2_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
 
 	if (! parser_is_oceanic_atom2 (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
 	parser->maxdepth = 0.0;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-oceanic_atom2_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime)
+static dc_status_t
+oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
 
@@ -154,7 +159,7 @@ oceanic_atom2_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime)
 		header = 32;
 
 	if (abstract->size < header)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	const unsigned char *p = abstract->data;
 
@@ -165,10 +170,12 @@ oceanic_atom2_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime)
 		switch (parser->model) {
 		case OC1A:
 		case OC1B:
+		case OCS:
 		case VT4:
 		case VT41:
 		case ATOM3:
 		case ATOM31:
+		case A300AI:
 			datetime->year   = ((p[5] & 0xE0) >> 5) + ((p[7] & 0xE0) >> 2) + 2000;
 			datetime->month  = (p[3] & 0x0F);
 			datetime->day    = ((p[0] & 0x80) >> 3) + ((p[3] & 0xF0) >> 4);
@@ -177,7 +184,9 @@ oceanic_atom2_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime)
 			break;
 		case VT3:
 		case VEO20:
+		case VEO30:
 		case GEO20:
+		case DG03:
 			datetime->year   = ((p[3] & 0xE0) >> 1) + (p[4] & 0x0F) + 2000;
 			datetime->month  = (p[4] & 0xF0) >> 4;
 			datetime->day    = p[3] & 0x1F;
@@ -249,12 +258,12 @@ oceanic_atom2_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime)
 		}
 	}
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, unsigned int flags, void *value)
+static dc_status_t
+oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
 {
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
 
@@ -266,7 +275,8 @@ oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, un
 	unsigned int footersize = 2 * PAGESIZE / 2;
 	if (parser->model == DATAMASK || parser->model == COMPUMASK ||
 		parser->model == GEO || parser->model == GEO20 ||
-		parser->model == VEO20 || parser->model == VEO30) {
+		parser->model == VEO20 || parser->model == VEO30 ||
+		parser->model == OCS) {
 		headersize -= PAGESIZE;
 	} else if (parser->model == VT4 || parser->model == VT41) {
 		headersize += PAGESIZE;
@@ -278,7 +288,7 @@ oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, un
 	}
 
 	if (size < headersize + footersize)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	// Get the offset to the header and footer sample.
 	unsigned int header = headersize - PAGESIZE / 2;
@@ -289,9 +299,9 @@ oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, un
 
 	if (!parser->cached) {
 		sample_statistics_t statistics = SAMPLE_STATISTICS_INITIALIZER;
-		parser_status_t rc = oceanic_atom2_parser_samples_foreach (
+		dc_status_t rc = oceanic_atom2_parser_samples_foreach (
 			abstract, sample_statistics_cb, &statistics);
-		if (rc != PARSER_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		parser->cached = 1;
@@ -299,25 +309,25 @@ oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, un
 		parser->maxdepth = statistics.maxdepth;
 	}
 
-	gasmix_t *gasmix = (gasmix_t *) value;
+	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 
 	unsigned int nitrox = 0;
 
 	if (value) {
 		switch (type) {
-		case FIELD_TYPE_DIVETIME:
+		case DC_FIELD_DIVETIME:
 			if (parser->model == F10)
 				*((unsigned int *) value) = bcd2dec (data[2]) + bcd2dec (data[3]) * 60 + bcd2dec (data[1]) * 3600;
 			else
 				*((unsigned int *) value) = parser->divetime;
 			break;
-		case FIELD_TYPE_MAXDEPTH:
+		case DC_FIELD_MAXDEPTH:
 			if (parser->model == F10)
 				*((double *) value) = array_uint16_le (data + 4) / 16.0 * FEET;
 			else
 				*((double *) value) = array_uint16_le (data + footer + 4) / 16.0 * FEET;
 			break;
-		case FIELD_TYPE_GASMIX_COUNT:
+		case DC_FIELD_GASMIX_COUNT:
 			if (parser->model == DATAMASK || parser->model == COMPUMASK)
 				*((unsigned int *) value) = 1;
 			else if (parser->model == VT4 || parser->model == VT41)
@@ -325,7 +335,7 @@ oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, un
 			else
 				*((unsigned int *) value) = 3;
 			break;
-		case FIELD_TYPE_GASMIX:
+		case DC_FIELD_GASMIX:
 			if (parser->model == DATAMASK || parser->model == COMPUMASK)
 				nitrox = data[header + 3];
 			else
@@ -335,21 +345,21 @@ oceanic_atom2_parser_get_field (parser_t *abstract, parser_field_type_t type, un
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
 			break;
 		default:
-			return PARSER_STATUS_UNSUPPORTED;
+			return DC_STATUS_UNSUPPORTED;
 		}
 	}
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t callback, void *userdata)
+static dc_status_t
+oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata)
 {
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
 
 	if (! parser_is_oceanic_atom2 (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
@@ -359,7 +369,8 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 	unsigned int footersize = 2 * PAGESIZE / 2;
 	if (parser->model == DATAMASK || parser->model == COMPUMASK ||
 		parser->model == GEO || parser->model == GEO20 ||
-		parser->model == VEO20 || parser->model == VEO30) {
+		parser->model == VEO20 || parser->model == VEO30 ||
+		parser->model == OCS) {
 		headersize -= PAGESIZE;
 	} else if (parser->model == VT4 || parser->model == VT41) {
 		headersize += PAGESIZE;
@@ -371,7 +382,7 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 	}
 
 	if (size < headersize + footersize)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	// Get the offset to the header sample.
 	unsigned int header = headersize - PAGESIZE / 2;
@@ -402,7 +413,7 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 		samplesize = 2;
 
 	unsigned int have_temperature = 1, have_pressure = 1;
-	if (parser->model == VEO30) {
+	if (parser->model == VEO30 || parser->model == OCS) {
 		have_pressure = 0;
 	} else if (parser->model == F10) {
 		have_temperature = 0;
@@ -427,7 +438,7 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 	unsigned int complete = 1;
 	unsigned int offset = headersize;
 	while (offset + samplesize <= size - footersize) {
-		parser_sample_value_t sample = {0};
+		dc_sample_value_t sample = {0};
 
 		// Ignore empty samples.
 		if (array_isequal (data + offset, samplesize, 0x00) ||
@@ -440,7 +451,7 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 		if (complete) {
 			time += interval;
 			sample.time = time;
-			if (callback) callback (SAMPLE_TYPE_TIME, sample, userdata);
+			if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
 
 			complete = 0;
 		}
@@ -456,14 +467,14 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 		if (sampletype == 0xBB) {
 			length = PAGESIZE;
 			if (offset + length > size - PAGESIZE)
-				return PARSER_STATUS_ERROR;
+				return DC_STATUS_DATAFORMAT;
 		}
 
 		// Vendor specific data
 		sample.vendor.type = SAMPLE_VENDOR_OCEANIC_ATOM2;
 		sample.vendor.size = length;
 		sample.vendor.data = data + offset;
-		if (callback) callback (SAMPLE_TYPE_VENDOR, sample, userdata);
+		if (callback) callback (DC_SAMPLE_VENDOR, sample, userdata);
 
 		// Check for a tank switch sample.
 		if (sampletype == 0xAA) {
@@ -474,7 +485,7 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 			} else {
 				// Tank pressure (2 psi) and number (one based index)
 				tank = (data[offset + 1] & 0x03) - 1;
-				if (parser->model == ATOM2 || parser->model == EPIC)
+				if (parser->model == ATOM2 || parser->model == EPICA || parser->model == EPICB)
 					pressure = (((data[offset + 3] << 8) + data[offset + 4]) & 0x0FFF) * 2;
 				else
 					pressure = (((data[offset + 4] << 8) + data[offset + 5]) & 0x0FFF) * 2;
@@ -490,11 +501,11 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 				if (complete) {
 					time += interval;
 					sample.time = time;
-					if (callback) callback (SAMPLE_TYPE_TIME, sample, userdata);
+					if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
 				}
 
 				sample.depth = 0.0;
-				if (callback) callback (SAMPLE_TYPE_DEPTH, sample, userdata);
+				if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 				complete = 1;
 			}
 		} else {
@@ -506,11 +517,16 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 					parser->model == VEO30 || parser->model == OC1A ||
 					parser->model == OC1B) {
 					temperature = data[offset + 3];
-				} else if (parser->model == VT4 || parser->model == VT41 || parser->model == ATOM3 || parser->model == ATOM31) {
+				} else if (parser->model == OCS) {
+					temperature = data[offset + 1];
+				} else if (parser->model == VT4 || parser->model == VT41 || parser->model == ATOM3 || parser->model == ATOM31 || parser->model == A300AI) {
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else {
 					unsigned int sign;
-					if (parser->model == ATOM2 || parser->model == EPIC || parser->model == PROPLUS21)
+					if (parser->model == DG03)
+						sign = (~data[offset + 5] & 0x04) >> 2;
+					else if (parser->model == ATOM2 || parser->model == PROPLUS21 ||
+						parser->model == EPICA || parser->model == EPICB)
 						sign = (data[offset + 0] & 0x80) >> 7;
 					else
 						sign = (~data[offset + 0] & 0x80) >> 7;
@@ -520,20 +536,23 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 						temperature += (data[offset + 7] & 0x0C) >> 2;
 				}
 				sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
-				if (callback) callback (SAMPLE_TYPE_TEMPERATURE, sample, userdata);
+				if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 			}
 
 			// Tank Pressure (psi)
 			if (have_pressure) {
 				if (parser->model == OC1A || parser->model == OC1B)
 					pressure = (data[offset + 10] + (data[offset + 11] << 8)) & 0x0FFF;
-				else if (parser->model == ZENAIR || parser->model == VT4 || parser->model == VT41|| parser->model == ATOM3 || parser->model == ATOM31)
+				else if (parser->model == VT4 || parser->model == VT41||
+					parser->model == ATOM3 || parser->model == ATOM31 ||
+					parser->model == ZENAIR ||parser->model == A300AI ||
+					parser->model == DG03)
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else
 					pressure -= data[offset + 1];
 				sample.pressure.tank = tank;
 				sample.pressure.value = pressure * PSI / BAR;
-				if (callback) callback (SAMPLE_TYPE_PRESSURE, sample, userdata);
+				if (callback) callback (DC_SAMPLE_PRESSURE, sample, userdata);
 			}
 
 			// Depth (1/16 ft)
@@ -549,7 +568,7 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 			else
 				depth = (data[offset + 2] + (data[offset + 3] << 8)) & 0x0FFF;
 			sample.depth = depth / 16.0 * FEET;
-			if (callback) callback (SAMPLE_TYPE_DEPTH, sample, userdata);
+			if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 
 			complete = 1;
 		}
@@ -557,5 +576,5 @@ oceanic_atom2_parser_samples_foreach (parser_t *abstract, sample_callback_t call
 		offset += length;
 	}
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
