@@ -24,10 +24,10 @@
 #include <assert.h> // assert
 
 #include "oceanic_common.h"
+#include "context-private.h"
 #include "device-private.h"
 #include "ringbuffer.h"
 #include "array.h"
-#include "utils.h"
 
 #define RB_LOGBOOK_DISTANCE(a,b,l)	ringbuffer_distance (a, b, 0, l->rb_logbook_begin, l->rb_logbook_end)
 #define RB_LOGBOOK_INCR(a,b,l)		ringbuffer_increment (a, b, l->rb_logbook_begin, l->rb_logbook_end)
@@ -92,10 +92,10 @@ get_profile_last (const unsigned char data[], const oceanic_common_layout_t *lay
 }
 
 
-int
-oceanic_common_match (const unsigned char *pattern, const unsigned char *string, unsigned int n)
+static int
+oceanic_common_match_pattern (const unsigned char *string, const unsigned char *pattern)
 {
-	for (unsigned int i = 0; i < n; ++i, ++pattern, ++string) {
+	for (unsigned int i = 0; i < PAGESIZE; ++i, ++pattern, ++string) {
 		if (*pattern != '\0' && *pattern != *string)
 			return 0;
 	}
@@ -104,23 +104,36 @@ oceanic_common_match (const unsigned char *pattern, const unsigned char *string,
 }
 
 
+int
+oceanic_common_match (const unsigned char *version, const oceanic_common_version_t patterns[], unsigned int n)
+{
+	for (unsigned int i = 0; i < n; ++i) {
+		if (oceanic_common_match_pattern (version, patterns[i]))
+			return 1;
+	}
+
+	return 0;
+}
+
+
 void
-oceanic_common_device_init (oceanic_common_device_t *device, const device_backend_t *backend)
+oceanic_common_device_init (oceanic_common_device_t *device, dc_context_t *context, const device_backend_t *backend)
 {
 	assert (device != NULL);
 
 	// Initialize the base class.
-	device_init (&device->base, backend);
+	device_init (&device->base, context, backend);
 
 	// Set the default values.
+	memset (device->version, 0, sizeof (device->version));
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 	device->layout = NULL;
 	device->multipage = 1;
 }
 
 
-device_status_t
-oceanic_common_device_set_fingerprint (device_t *abstract, const unsigned char data[], unsigned int size)
+dc_status_t
+oceanic_common_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
 	oceanic_common_device_t *device = (oceanic_common_device_t *) abstract;
 
@@ -131,19 +144,19 @@ oceanic_common_device_set_fingerprint (device_t *abstract, const unsigned char d
 	unsigned int fpsize = device->layout->rb_logbook_entry_size;
 
 	if (size && size != fpsize)
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	if (size)
 		memcpy (device->fingerprint, data, fpsize);
 	else
 		memset (device->fingerprint, 0, fpsize);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-oceanic_common_device_dump (device_t *abstract, dc_buffer_t *buffer)
+dc_status_t
+oceanic_common_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	oceanic_common_device_t *device = (oceanic_common_device_t *) abstract;
 
@@ -153,17 +166,23 @@ oceanic_common_device_dump (device_t *abstract, dc_buffer_t *buffer)
 	// Erase the current contents of the buffer and
 	// allocate the required amount of memory.
 	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, device->layout->memsize)) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_NOMEMORY;
 	}
 
+	// Emit a vendor event.
+	dc_event_vendor_t vendor;
+	vendor.data = device->version;
+	vendor.size = sizeof (device->version);
+	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
+
 	return device_dump_read (abstract, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), PAGESIZE * device->multipage);
 }
 
 
-device_status_t
-oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, void *userdata)
+dc_status_t
+oceanic_common_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
 	oceanic_common_device_t *device = (oceanic_common_device_t *) abstract;
 
@@ -174,39 +193,45 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 	const oceanic_common_layout_t *layout = device->layout;
 
 	// Enable progress notifications.
-	device_progress_t progress = DEVICE_PROGRESS_INITIALIZER;
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	progress.maximum = 2 * PAGESIZE +
 		(layout->rb_profile_end - layout->rb_profile_begin) +
 		(layout->rb_logbook_end - layout->rb_logbook_begin);
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
+
+	// Emit a vendor event.
+	dc_event_vendor_t vendor;
+	vendor.data = device->version;
+	vendor.size = sizeof (device->version);
+	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
 
 	// Read the device id.
 	unsigned char id[PAGESIZE] = {0};
-	device_status_t rc = device_read (abstract, layout->cf_devinfo, id, sizeof (id));
-	if (rc != DEVICE_STATUS_SUCCESS) {
-		WARNING ("Cannot read device id.");
+	dc_status_t rc = dc_device_read (abstract, layout->cf_devinfo, id, sizeof (id));
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the memory page.");
 		return rc;
 	}
 
 	// Update and emit a progress event.
 	progress.current += PAGESIZE;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Emit a device info event.
-	device_devinfo_t devinfo;
+	dc_event_devinfo_t devinfo;
 	devinfo.model = array_uint16_be (id + 8);
 	devinfo.firmware = 0;
 	if (layout->pt_mode_global == 0)
 		devinfo.serial = bcd2dec (id[10]) * 10000 + bcd2dec (id[11]) * 100 + bcd2dec (id[12]);
 	else
 		devinfo.serial = id[11] * 10000 + id[12] * 100 + id[13];
-	device_event_emit (abstract, DEVICE_EVENT_DEVINFO, &devinfo);
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	// Read the pointer data.
 	unsigned char pointers[PAGESIZE] = {0};
-	rc = device_read (abstract, layout->cf_pointers, pointers, sizeof (pointers));
-	if (rc != DEVICE_STATUS_SUCCESS) {
-		WARNING ("Cannot read pointers.");
+	rc = dc_device_read (abstract, layout->cf_pointers, pointers, sizeof (pointers));
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the memory page.");
 		return rc;
 	}
 
@@ -276,12 +301,12 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 	progress.maximum = 2 * PAGESIZE +
 		(layout->rb_profile_end - layout->rb_profile_begin) +
 		rb_logbook_page_size;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Memory buffer for the logbook entries.
 	unsigned char *logbooks = (unsigned char *) malloc (rb_logbook_page_size);
 	if (logbooks == NULL)
-		return DEVICE_STATUS_MEMORY;
+		return DC_STATUS_NOMEMORY;
 
 	// Since entries are not necessary aligned on page boundaries,
 	// the memory buffer may contain padding entries on both sides.
@@ -295,7 +320,7 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 	}
 
 	// Error status for delayed errors.
-	device_status_t status = DEVICE_STATUS_SUCCESS;
+	dc_status_t status = DC_STATUS_SUCCESS;
 
 	// Keep track of the previous dive.
 	unsigned int remaining = layout->rb_profile_end - layout->rb_profile_begin;
@@ -327,15 +352,15 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 		offset -= len;
 
 		// Read the logbook page.
-		rc = device_read (abstract, address, logbooks + offset, len);
-		if (rc != DEVICE_STATUS_SUCCESS) {
+		rc = dc_device_read (abstract, address, logbooks + offset, len);
+		if (rc != DC_STATUS_SUCCESS) {
 			free (logbooks);
 			return rc;
 		}
 
 		// Update and emit a progress event.
 		progress.current += len;
-		device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 		// A full ringbuffer needs some special treatment to avoid
 		// having to download the first/last page twice. When a full
@@ -376,7 +401,7 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 			// This appears to happen on some devices, and we attempt to
 			// fix this here.
 			if (array_isequal (logbooks + current, layout->rb_logbook_entry_size, 0xFF)) {
-				WARNING("Uninitialized logbook entries detected!");
+				WARNING (abstract->context, "Uninitialized logbook entries detected!");
 				begin = current + layout->rb_logbook_entry_size;
 				abort = 1;
 				break;
@@ -394,8 +419,8 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 				rb_entry_last < layout->rb_profile_begin ||
 				rb_entry_last >= layout->rb_profile_end)
 			{
-				WARNING("Invalid ringbuffer pointer detected!");
-				status = DEVICE_STATUS_ERROR;
+				ERROR (abstract->context, "Invalid ringbuffer pointer detected.");
+				status = DC_STATUS_DATAFORMAT;
 				begin = current + layout->rb_logbook_entry_size;
 				abort = 1;
 				break;
@@ -408,13 +433,13 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 			// Skip gaps between the profiles.
 			unsigned int gap = 0;
 			if (previous && rb_entry_end != previous) {
-				WARNING ("Profiles are not continuous.");
+				WARNING (abstract->context, "Profiles are not continuous.");
 				gap = RB_PROFILE_DISTANCE (rb_entry_end, previous, layout);
 			}
 
 			// Make sure the profile size is valid.
 			if (rb_entry_size + gap > remaining) {
-				WARNING ("Unexpected profile size.");
+				WARNING (abstract->context, "Unexpected profile size.");
 				begin = current + layout->rb_logbook_entry_size;
 				abort = 1;
 				break;
@@ -457,7 +482,7 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 	unsigned char *profiles = (unsigned char *) malloc (rb_profile_size + (end - begin));
 	if (profiles == NULL) {
 		free (logbooks);
-		return DEVICE_STATUS_MEMORY;
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// When using multipage reads, the last packet can contain data from more
@@ -489,7 +514,7 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 		// Skip gaps between the profiles.
 		unsigned int gap = 0;
 		if (rb_entry_end != previous) {
-			WARNING ("Profiles are not continuous.");
+			WARNING (abstract->context, "Profiles are not continuous.");
 			gap = RB_PROFILE_DISTANCE (rb_entry_end, previous, layout);
 		}
 
@@ -512,8 +537,8 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 			offset -= len;
 
 			// Read the profile page.
-			rc = device_read (abstract, address, profiles + offset, len);
-			if (rc != DEVICE_STATUS_SUCCESS) {
+			rc = dc_device_read (abstract, address, profiles + offset, len);
+			if (rc != DC_STATUS_SUCCESS) {
 				free (logbooks);
 				free (profiles);
 				return rc;
@@ -521,7 +546,7 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 
 			// Update and emit a progress event.
 			progress.current += len;
-			device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+			device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 			nbytes += len;
 		}
@@ -542,7 +567,7 @@ oceanic_common_device_foreach (device_t *abstract, dive_callback_t callback, voi
 		if (callback && !callback (p, rb_entry_size + layout->rb_logbook_entry_size, p, layout->rb_logbook_entry_size, userdata)) {
 			free (logbooks);
 			free (profiles);
-			return DEVICE_STATUS_SUCCESS;
+			return DC_STATUS_SUCCESS;
 		}
 	}
 
