@@ -31,6 +31,10 @@ extern "C" {
 #define PAGESIZE 0x10
 #define FPMAXSIZE 0x20
 
+#define OCEANIC_COMMON_MATCH(version,patterns) \
+	oceanic_common_match ((version), (patterns), \
+	sizeof (patterns) / sizeof *(patterns))
+
 typedef struct oceanic_common_layout_t {
 	// Memory size.
 	unsigned int memsize;
@@ -54,26 +58,29 @@ typedef struct oceanic_common_layout_t {
 } oceanic_common_layout_t;
 
 typedef struct oceanic_common_device_t {
-	device_t base;
+	dc_device_t base;
+	unsigned char version[PAGESIZE];
 	unsigned char fingerprint[FPMAXSIZE];
 	const oceanic_common_layout_t *layout;
 	unsigned int multipage;
 } oceanic_common_device_t;
 
+typedef unsigned char oceanic_common_version_t[PAGESIZE + 1];
+
 int
-oceanic_common_match (const unsigned char *pattern, const unsigned char *string, unsigned int n);
+oceanic_common_match (const unsigned char *version, const oceanic_common_version_t patterns[], unsigned int n);
 
 void
-oceanic_common_device_init (oceanic_common_device_t *device, const device_backend_t *backend);
+oceanic_common_device_init (oceanic_common_device_t *device, dc_context_t *context, const device_backend_t *backend);
 
-device_status_t
-oceanic_common_device_set_fingerprint (device_t *device, const unsigned char data[], unsigned int size);
+dc_status_t
+oceanic_common_device_set_fingerprint (dc_device_t *device, const unsigned char data[], unsigned int size);
 
-device_status_t
-oceanic_common_device_dump (device_t *abstract, dc_buffer_t *buffer);
+dc_status_t
+oceanic_common_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
 
-device_status_t
-oceanic_common_device_foreach (device_t *device, dive_callback_t callback, void *userdata);
+dc_status_t
+oceanic_common_device_foreach (dc_device_t *device, dc_dive_callback_t callback, void *userdata);
 
 #ifdef __cplusplus
 }
