@@ -22,16 +22,17 @@
 #include <stdlib.h>
 #include <string.h>	// memcmp
 
-#include "reefnet_sensuspro.h"
+#include <libdivecomputer/reefnet_sensuspro.h>
+#include <libdivecomputer/units.h>
+
+#include "context-private.h"
 #include "parser-private.h"
-#include "units.h"
-#include "utils.h"
 #include "array.h"
 
 typedef struct reefnet_sensuspro_parser_t reefnet_sensuspro_parser_t;
 
 struct reefnet_sensuspro_parser_t {
-	parser_t base;
+	dc_parser_t base;
 	// Depth calibration.
 	double atmospheric;
 	double hydrostatic;
@@ -44,14 +45,14 @@ struct reefnet_sensuspro_parser_t {
 	unsigned int maxdepth;
 };
 
-static parser_status_t reefnet_sensuspro_parser_set_data (parser_t *abstract, const unsigned char *data, unsigned int size);
-static parser_status_t reefnet_sensuspro_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime);
-static parser_status_t reefnet_sensuspro_parser_get_field (parser_t *abstract, parser_field_type_t type, unsigned int flags, void *value);
-static parser_status_t reefnet_sensuspro_parser_samples_foreach (parser_t *abstract, sample_callback_t callback, void *userdata);
-static parser_status_t reefnet_sensuspro_parser_destroy (parser_t *abstract);
+static dc_status_t reefnet_sensuspro_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
+static dc_status_t reefnet_sensuspro_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime);
+static dc_status_t reefnet_sensuspro_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
+static dc_status_t reefnet_sensuspro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
+static dc_status_t reefnet_sensuspro_parser_destroy (dc_parser_t *abstract);
 
 static const parser_backend_t reefnet_sensuspro_parser_backend = {
-	PARSER_TYPE_REEFNET_SENSUSPRO,
+	DC_FAMILY_REEFNET_SENSUSPRO,
 	reefnet_sensuspro_parser_set_data, /* set_data */
 	reefnet_sensuspro_parser_get_datetime, /* datetime */
 	reefnet_sensuspro_parser_get_field, /* fields */
@@ -61,7 +62,7 @@ static const parser_backend_t reefnet_sensuspro_parser_backend = {
 
 
 static int
-parser_is_reefnet_sensuspro (parser_t *abstract)
+parser_is_reefnet_sensuspro (dc_parser_t *abstract)
 {
 	if (abstract == NULL)
 		return 0;
@@ -70,21 +71,21 @@ parser_is_reefnet_sensuspro (parser_t *abstract)
 }
 
 
-parser_status_t
-reefnet_sensuspro_parser_create (parser_t **out, unsigned int devtime, dc_ticks_t systime)
+dc_status_t
+reefnet_sensuspro_parser_create (dc_parser_t **out, dc_context_t *context, unsigned int devtime, dc_ticks_t systime)
 {
 	if (out == NULL)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t *) malloc (sizeof (reefnet_sensuspro_parser_t));
 	if (parser == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return PARSER_STATUS_MEMORY;
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, &reefnet_sensuspro_parser_backend);
+	parser_init (&parser->base, context, &reefnet_sensuspro_parser_backend);
 
 	// Set the default values.
 	parser->atmospheric = ATM;
@@ -95,83 +96,83 @@ reefnet_sensuspro_parser_create (parser_t **out, unsigned int devtime, dc_ticks_
 	parser->divetime = 0;
 	parser->maxdepth = 0;
 
-	*out = (parser_t*) parser;
+	*out = (dc_parser_t*) parser;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-reefnet_sensuspro_parser_destroy (parser_t *abstract)
+static dc_status_t
+reefnet_sensuspro_parser_destroy (dc_parser_t *abstract)
 {
 	if (! parser_is_reefnet_sensuspro (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Free memory.	
 	free (abstract);
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-reefnet_sensuspro_parser_set_data (parser_t *abstract, const unsigned char *data, unsigned int size)
+static dc_status_t
+reefnet_sensuspro_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t*) abstract;
 
 	if (! parser_is_reefnet_sensuspro (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
 	parser->maxdepth = 0;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-parser_status_t
-reefnet_sensuspro_parser_set_calibration (parser_t *abstract, double atmospheric, double hydrostatic)
+dc_status_t
+reefnet_sensuspro_parser_set_calibration (dc_parser_t *abstract, double atmospheric, double hydrostatic)
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t*) abstract;
 
 	if (! parser_is_reefnet_sensuspro (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	parser->atmospheric = atmospheric;
 	parser->hydrostatic = hydrostatic;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-reefnet_sensuspro_parser_get_datetime (parser_t *abstract, dc_datetime_t *datetime)
+static dc_status_t
+reefnet_sensuspro_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetime)
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t *) abstract;
 
 	if (abstract->size < 6 + 4)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	unsigned int timestamp = array_uint32_le (abstract->data + 6);
 
 	dc_ticks_t ticks = parser->systime - (parser->devtime - timestamp);
 
 	if (!dc_datetime_localtime (datetime, ticks))
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-reefnet_sensuspro_parser_get_field (parser_t *abstract, parser_field_type_t type, unsigned int flags, void *value)
+static dc_status_t
+reefnet_sensuspro_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t *) abstract;
 
 	if (abstract->size < 12)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	if (!parser->cached) {
 		const unsigned char footer[2] = {0xFF, 0xFF};
@@ -204,31 +205,31 @@ reefnet_sensuspro_parser_get_field (parser_t *abstract, parser_field_type_t type
 
 	if (value) {
 		switch (type) {
-		case FIELD_TYPE_DIVETIME:
+		case DC_FIELD_DIVETIME:
 			*((unsigned int *) value) = parser->divetime;
 			break;
-		case FIELD_TYPE_MAXDEPTH:
+		case DC_FIELD_MAXDEPTH:
 			*((double *) value) = (parser->maxdepth * FSW - parser->atmospheric) / parser->hydrostatic;
 			break;
-		case FIELD_TYPE_GASMIX_COUNT:
+		case DC_FIELD_GASMIX_COUNT:
 			*((unsigned int *) value) = 0;
 			break;
 		default:
-			return PARSER_STATUS_UNSUPPORTED;
+			return DC_STATUS_UNSUPPORTED;
 		}
 	}
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-reefnet_sensuspro_parser_samples_foreach (parser_t *abstract, sample_callback_t callback, void *userdata)
+static dc_status_t
+reefnet_sensuspro_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata)
 {
 	reefnet_sensuspro_parser_t *parser = (reefnet_sensuspro_parser_t*) abstract;
 
 	if (! parser_is_reefnet_sensuspro (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	const unsigned char header[4] = {0x00, 0x00, 0x00, 0x00};
 	const unsigned char footer[2] = {0xFF, 0xFF};
@@ -240,7 +241,7 @@ reefnet_sensuspro_parser_samples_foreach (parser_t *abstract, sample_callback_t
 	while (offset + sizeof (header) <= size) {
 		if (memcmp (data + offset, header, sizeof (header)) == 0) {
 			if (offset + 10 > size)
-				return PARSER_STATUS_ERROR;
+				return DC_STATUS_DATAFORMAT;
 
 			unsigned int time = 0;
 			unsigned int interval = array_uint16_le (data + offset + 4);	
@@ -253,20 +254,20 @@ reefnet_sensuspro_parser_samples_foreach (parser_t *abstract, sample_callback_t
 				unsigned int depth = (value & 0x01FF);
 				unsigned int temperature = (value & 0xFE00) >> 9;
 
-				parser_sample_value_t sample = {0};
+				dc_sample_value_t sample = {0};
 
 				// Time (seconds)
 				time += interval;
 				sample.time = time;
-				if (callback) callback (SAMPLE_TYPE_TIME, sample, userdata);
+				if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
 
 				// Temperature (°F)
 				sample.temperature = (temperature - 32.0) * (5.0 / 9.0);
-				if (callback) callback (SAMPLE_TYPE_TEMPERATURE, sample, userdata);
+				if (callback) callback (DC_SAMPLE_TEMPERATURE, sample, userdata);
 
 				// Depth (absolute pressure in fsw)
 				sample.depth = (depth * FSW - parser->atmospheric) / parser->hydrostatic;
-				if (callback) callback (SAMPLE_TYPE_DEPTH, sample, userdata);
+				if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 
 				offset += 2;
 			}
@@ -276,5 +277,5 @@ reefnet_sensuspro_parser_samples_foreach (parser_t *abstract, sample_callback_t
 		}
 	}
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
