@@ -23,41 +23,47 @@
 #include <stdlib.h> // malloc, free
 #include <assert.h> // assert
 
+#include <libdivecomputer/reefnet_sensusultra.h>
+
+#include "context-private.h"
 #include "device-private.h"
-#include "reefnet_sensusultra.h"
 #include "serial.h"
 #include "checksum.h"
-#include "utils.h"
 #include "array.h"
 
 #define EXITCODE(rc) \
 ( \
-	rc == -1 ? DEVICE_STATUS_IO : DEVICE_STATUS_TIMEOUT \
+	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
 )
 
+#define SZ_PACKET    512
+#define SZ_MEMORY    2080768
+#define SZ_USER      16384
+#define SZ_HANDSHAKE 24
+#define SZ_SENSE     6
+
+#define MAXRETRIES 2
 #define PROMPT 0xA5
 #define ACCEPT PROMPT
 #define REJECT 0x00
 
 typedef struct reefnet_sensusultra_device_t {
-	device_t base;
+	dc_device_t base;
 	serial_t *port;
-	unsigned char handshake[REEFNET_SENSUSULTRA_HANDSHAKE_SIZE];
-	unsigned int maxretries;
+	unsigned char handshake[SZ_HANDSHAKE];
 	unsigned int timestamp;
 	unsigned int devtime;
 	dc_ticks_t systime;
 } reefnet_sensusultra_device_t;
 
-static device_status_t reefnet_sensusultra_device_set_fingerprint (device_t *abstract, const unsigned char data[], unsigned int size);
-static device_status_t reefnet_sensusultra_device_dump (device_t *abstract, dc_buffer_t *buffer);
-static device_status_t reefnet_sensusultra_device_foreach (device_t *abstract, dive_callback_t callback, void *userdata);
-static device_status_t reefnet_sensusultra_device_close (device_t *abstract);
+static dc_status_t reefnet_sensusultra_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size);
+static dc_status_t reefnet_sensusultra_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
+static dc_status_t reefnet_sensusultra_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
+static dc_status_t reefnet_sensusultra_device_close (dc_device_t *abstract);
 
 static const device_backend_t reefnet_sensusultra_device_backend = {
-	DEVICE_TYPE_REEFNET_SENSUSULTRA,
+	DC_FAMILY_REEFNET_SENSUSULTRA,
 	reefnet_sensusultra_device_set_fingerprint, /* set_fingerprint */
-	NULL, /* version */
 	NULL, /* read */
 	NULL, /* write */
 	reefnet_sensusultra_device_dump, /* dump */
@@ -66,7 +72,7 @@ static const device_backend_t reefnet_sensusultra_device_backend = {
 };
 
 static int
-device_is_reefnet_sensusultra (device_t *abstract)
+device_is_reefnet_sensusultra (dc_device_t *abstract)
 {
 	if (abstract == NULL)
 		return 0;
@@ -75,213 +81,186 @@ device_is_reefnet_sensusultra (device_t *abstract)
 }
 
 
-device_status_t
-reefnet_sensusultra_device_open (device_t **out, const char* name)
+dc_status_t
+reefnet_sensusultra_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
 	if (out == NULL)
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t *) malloc (sizeof (reefnet_sensusultra_device_t));
 	if (device == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	device_init (&device->base, &reefnet_sensusultra_device_backend);
+	device_init (&device->base, context, &reefnet_sensusultra_device_backend);
 
 	// Set the default values.
 	device->port = NULL;
-	device->maxretries = 2;
 	device->timestamp = 0;
 	device->systime = (dc_ticks_t) -1;
 	device->devtime = 0;
 	memset (device->handshake, 0, sizeof (device->handshake));
 
 	// Open the device.
-	int rc = serial_open (&device->port, name);
+	int rc = serial_open (&device->port, context, name);
 	if (rc == -1) {
-		WARNING ("Failed to open the serial port.");
+		ERROR (context, "Failed to open the serial port.");
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Set the serial communication protocol (115200 8N1).
 	rc = serial_configure (device->port, 115200, 8, SERIAL_PARITY_NONE, 1, SERIAL_FLOWCONTROL_NONE);
 	if (rc == -1) {
-		WARNING ("Failed to set the terminal attributes.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Set the timeout for receiving data (3000ms).
 	if (serial_set_timeout (device->port, 3000) == -1) {
-		WARNING ("Failed to set the timeout.");
+		ERROR (context, "Failed to set the timeout.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Make sure everything is in a sane state.
 	serial_flush (device->port, SERIAL_QUEUE_BOTH);
 
-	*out = (device_t*) device;
+	*out = (dc_device_t*) device;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-reefnet_sensusultra_device_close (device_t *abstract)
+static dc_status_t
+reefnet_sensusultra_device_close (dc_device_t *abstract)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Free memory.	
 	free (device);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_device_get_handshake (device_t *abstract, unsigned char data[], unsigned int size)
+dc_status_t
+reefnet_sensusultra_device_get_handshake (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
-	if (size < REEFNET_SENSUSULTRA_HANDSHAKE_SIZE) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+	if (size < SZ_HANDSHAKE) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_INVALIDARGS;
 	}
 
-	memcpy (data, device->handshake, REEFNET_SENSUSULTRA_HANDSHAKE_SIZE);
+	memcpy (data, device->handshake, SZ_HANDSHAKE);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_device_set_maxretries (device_t *abstract, unsigned int maxretries)
+static dc_status_t
+reefnet_sensusultra_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
-
-	device->maxretries = maxretries;
-
-	return DEVICE_STATUS_SUCCESS;
-}
-
-
-device_status_t
-reefnet_sensusultra_device_set_timestamp (device_t *abstract, unsigned int timestamp)
-{
-	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
-
-	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
-
-	device->timestamp = timestamp;
-
-	return DEVICE_STATUS_SUCCESS;
-}
-
-
-static device_status_t
-reefnet_sensusultra_device_set_fingerprint (device_t *abstract, const unsigned char data[], unsigned int size)
-{
-	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
-
-	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	if (size && size != 4)
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	if (size)
 		device->timestamp = array_uint32_le (data);
 	else
 		device->timestamp = 0;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_send_uchar (reefnet_sensusultra_device_t *device, unsigned char value)
 {
+	dc_device_t *abstract = (dc_device_t *) device;
+
 	// Wait for the prompt byte.
 	unsigned char prompt = 0;
 	int rc = serial_read (device->port, &prompt, 1);
 	if (rc != 1) {
-		WARNING ("Failed to receive the prompt byte");
+		ERROR (abstract->context, "Failed to receive the prompt byte");
 		return EXITCODE (rc);
 	}
 
 	// Verify the prompt byte.
 	if (prompt != PROMPT) {
-		WARNING ("Unexpected answer data.");
-		return DEVICE_STATUS_PROTOCOL;
+		ERROR (abstract->context, "Unexpected answer data.");
+		return DC_STATUS_PROTOCOL;
 	}
 
 	// Send the value to the device.
 	rc = serial_write (device->port, &value, 1);
 	if (rc != 1) {
-		WARNING ("Failed to send the value.");
+		ERROR (abstract->context, "Failed to send the value.");
 		return EXITCODE (rc);
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_send_ushort (reefnet_sensusultra_device_t *device, unsigned short value)
 {
 	// Send the least-significant byte.
 	unsigned char lsb = value & 0xFF;
-	device_status_t rc = reefnet_sensusultra_send_uchar (device, lsb);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = reefnet_sensusultra_send_uchar (device, lsb);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	// Send the most-significant byte.
 	unsigned char msb = (value >> 8) & 0xFF;
 	rc = reefnet_sensusultra_send_uchar (device, msb);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_packet (reefnet_sensusultra_device_t *device, unsigned char *data, unsigned int size, unsigned int header)
 {
 	assert (size >= header + 2);
 
-	device_t *abstract = (device_t *) device;
+	dc_device_t *abstract = (dc_device_t *) device;
 
 	if (device_is_cancelled (abstract))
-		return DEVICE_STATUS_CANCELLED;
+		return DC_STATUS_CANCELLED;
 
 	// Receive the data packet.
 	int rc = serial_read (device->port, data, size);
 	if (rc != size) {
-		WARNING ("Failed to receive the packet.");
+		ERROR (abstract->context, "Failed to receive the packet.");
 		return EXITCODE (rc);
 	}
 
@@ -289,21 +268,21 @@ reefnet_sensusultra_packet (reefnet_sensusultra_device_t *device, unsigned char
 	unsigned short crc = array_uint16_le (data + size - 2);
 	unsigned short ccrc = checksum_crc_ccitt_uint16 (data + header, size - header - 2);
 	if (crc != ccrc) {
-		WARNING ("Unexpected answer CRC.");
-		return DEVICE_STATUS_PROTOCOL;
+		ERROR (abstract->context, "Unexpected answer checksum.");
+		return DC_STATUS_PROTOCOL;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_handshake (reefnet_sensusultra_device_t *device, unsigned short value)
 {
 	// Wake-up the device.
-	unsigned char handshake[REEFNET_SENSUSULTRA_HANDSHAKE_SIZE + 2] = {0};
-	device_status_t rc = reefnet_sensusultra_packet (device, handshake, sizeof (handshake), 0);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	unsigned char handshake[SZ_HANDSHAKE + 2] = {0};
+	dc_status_t rc = reefnet_sensusultra_packet (device, handshake, sizeof (handshake), 0);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	// Store the clock calibration values.
@@ -311,65 +290,73 @@ reefnet_sensusultra_handshake (reefnet_sensusultra_device_t *device, unsigned sh
 	device->devtime = array_uint32_le (handshake + 4);
 
 	// Store the handshake packet.
-	memcpy (device->handshake, handshake, REEFNET_SENSUSULTRA_HANDSHAKE_SIZE);
+	memcpy (device->handshake, handshake, SZ_HANDSHAKE);
 
 	// Emit a clock event.
-	device_clock_t clock;
+	dc_event_clock_t clock;
 	clock.systime = device->systime;
 	clock.devtime = device->devtime;
-	device_event_emit (&device->base, DEVICE_EVENT_CLOCK, &clock);
+	device_event_emit (&device->base, DC_EVENT_CLOCK, &clock);
 
 	// Emit a device info event.
-	device_devinfo_t devinfo;
+	dc_event_devinfo_t devinfo;
 	devinfo.model = handshake[1];
 	devinfo.firmware = handshake[0];
 	devinfo.serial = array_uint16_le (handshake + 2);
-	device_event_emit (&device->base, DEVICE_EVENT_DEVINFO, &devinfo);
+	device_event_emit (&device->base, DC_EVENT_DEVINFO, &devinfo);
+
+	// Emit a vendor event.
+	dc_event_vendor_t vendor;
+	vendor.data = device->handshake;
+	vendor.size = sizeof (device->handshake);
+	device_event_emit (&device->base, DC_EVENT_VENDOR, &vendor);
 
 	// Send the instruction code to the device.
 	rc = reefnet_sensusultra_send_ushort (device, value);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_page (reefnet_sensusultra_device_t *device, unsigned char *data, unsigned int size, unsigned int pagenum)
 {
-	assert (size >= REEFNET_SENSUSULTRA_PACKET_SIZE + 4);
+	dc_device_t *abstract = (dc_device_t *) device;
+
+	assert (size >= SZ_PACKET + 4);
 
 	unsigned int nretries = 0;
-	device_status_t rc = DEVICE_STATUS_SUCCESS;
-	while ((rc = reefnet_sensusultra_packet (device, data, size, 2)) != DEVICE_STATUS_SUCCESS) {
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	while ((rc = reefnet_sensusultra_packet (device, data, size, 2)) != DC_STATUS_SUCCESS) {
 		// Automatically discard a corrupted packet, 
 		// and request a new one.
-		if (rc != DEVICE_STATUS_PROTOCOL)
+		if (rc != DC_STATUS_PROTOCOL)
 			return rc;
 
 		// Abort if the maximum number of retries is reached.
-		if (nretries++ >= device->maxretries)
+		if (nretries++ >= MAXRETRIES)
 			return rc;
 
 		// Reject the packet.
 		rc = reefnet_sensusultra_send_uchar (device, REJECT);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 	}
 
 	// Verify the page number.
 	unsigned int page = array_uint16_le (data);
 	if (page != pagenum) {
-		WARNING ("Unexpected page number."); 
-		return DEVICE_STATUS_PROTOCOL;
+		ERROR (abstract->context, "Unexpected page number.");
+		return DC_STATUS_PROTOCOL;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_send (reefnet_sensusultra_device_t *device, unsigned short command)
 {
 	// Flush the input and output buffers.
@@ -377,15 +364,15 @@ reefnet_sensusultra_send (reefnet_sensusultra_device_t *device, unsigned short c
 
 	// Wake-up the device and send the instruction code.
 	unsigned int nretries = 0;
-	device_status_t rc = DEVICE_STATUS_SUCCESS;
-	while ((rc = reefnet_sensusultra_handshake (device, command)) != DEVICE_STATUS_SUCCESS) {
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	while ((rc = reefnet_sensusultra_handshake (device, command)) != DC_STATUS_SUCCESS) {
 		// Automatically discard a corrupted handshake packet,
 		// and wait for the next one.
-		if (rc != DEVICE_STATUS_PROTOCOL && rc != DEVICE_STATUS_TIMEOUT)
+		if (rc != DC_STATUS_PROTOCOL && rc != DC_STATUS_TIMEOUT)
 			return rc;
 
 		// Abort if the maximum number of retries is reached.
-		if (nretries++ >= device->maxretries)
+		if (nretries++ >= MAXRETRIES)
 			return rc;
 
 		// According to the developers guide, a 250 ms delay is suggested to
@@ -393,169 +380,169 @@ reefnet_sensusultra_send (reefnet_sensusultra_device_t *device, unsigned short c
 		// not accidentally buffered by the host and (mis)interpreted as part
 		// of the next packet.
 
-		serial_sleep (250);
+		serial_sleep (device->port, 250);
 		serial_flush (device->port, SERIAL_QUEUE_BOTH);
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-reefnet_sensusultra_device_dump (device_t *abstract, dc_buffer_t *buffer)
+static dc_status_t
+reefnet_sensusultra_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Erase the current contents of the buffer and
 	// pre-allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, REEFNET_SENSUSULTRA_MEMORY_DATA_SIZE)) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, SZ_MEMORY)) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Enable progress notifications.
-	device_progress_t progress = DEVICE_PROGRESS_INITIALIZER;
-	progress.maximum = REEFNET_SENSUSULTRA_MEMORY_DATA_SIZE;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	progress.maximum = SZ_MEMORY;
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Wake-up the device and send the instruction code.
-	device_status_t rc = reefnet_sensusultra_send (device, 0xB421);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = reefnet_sensusultra_send (device, 0xB421);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	unsigned int nbytes = 0;
 	unsigned int npages = 0;
-	while (nbytes < REEFNET_SENSUSULTRA_MEMORY_DATA_SIZE) {
+	while (nbytes < SZ_MEMORY) {
 		// Receive the packet.
-		unsigned char packet[REEFNET_SENSUSULTRA_PACKET_SIZE + 4] = {0};
+		unsigned char packet[SZ_PACKET + 4] = {0};
 		rc = reefnet_sensusultra_page (device, packet, sizeof (packet), npages);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		// Update and emit a progress event.
-		progress.current += REEFNET_SENSUSULTRA_PACKET_SIZE;
-		device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+		progress.current += SZ_PACKET;
+		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 		// Prepend the packet to the buffer.
-		if (!dc_buffer_prepend (buffer, packet + 2, REEFNET_SENSUSULTRA_PACKET_SIZE)) {
-			WARNING ("Insufficient buffer space available.");
-			return DEVICE_STATUS_MEMORY;
+		if (!dc_buffer_prepend (buffer, packet + 2, SZ_PACKET)) {
+			ERROR (abstract->context, "Insufficient buffer space available.");
+			return DC_STATUS_NOMEMORY;
 		}
 
 		// Accept the packet.
 		rc = reefnet_sensusultra_send_uchar (device, ACCEPT);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
-		nbytes += REEFNET_SENSUSULTRA_PACKET_SIZE;
+		nbytes += SZ_PACKET;
 		npages++;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_device_read_user (device_t *abstract, unsigned char *data, unsigned int size)
+dc_status_t
+reefnet_sensusultra_device_read_user (dc_device_t *abstract, unsigned char *data, unsigned int size)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
-	if (size < REEFNET_SENSUSULTRA_MEMORY_USER_SIZE) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+	if (size < SZ_USER) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_INVALIDARGS;
 	}
 
 	// Wake-up the device and send the instruction code.
-	device_status_t rc = reefnet_sensusultra_send (device, 0xB420);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = reefnet_sensusultra_send (device, 0xB420);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	unsigned int nbytes = 0;
 	unsigned int npages = 0;
-	while (nbytes < REEFNET_SENSUSULTRA_MEMORY_USER_SIZE) {
+	while (nbytes < SZ_USER) {
 		// Receive the packet.
-		unsigned char packet[REEFNET_SENSUSULTRA_PACKET_SIZE + 4] = {0};
+		unsigned char packet[SZ_PACKET + 4] = {0};
 		rc = reefnet_sensusultra_page (device, packet, sizeof (packet), npages);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		// Append the packet to the buffer.
-		memcpy (data + nbytes, packet + 2, REEFNET_SENSUSULTRA_PACKET_SIZE);
+		memcpy (data + nbytes, packet + 2, SZ_PACKET);
 
 		// Accept the packet.
 		rc = reefnet_sensusultra_send_uchar (device, ACCEPT);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
-		nbytes += REEFNET_SENSUSULTRA_PACKET_SIZE;
+		nbytes += SZ_PACKET;
 		npages++;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_device_write_user (device_t *abstract, const unsigned char *data, unsigned int size)
+dc_status_t
+reefnet_sensusultra_device_write_user (dc_device_t *abstract, const unsigned char *data, unsigned int size)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
-	if (size < REEFNET_SENSUSULTRA_MEMORY_USER_SIZE) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+	if (size < SZ_USER) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_INVALIDARGS;
 	}
 
 	// Enable progress notifications.
-	device_progress_t progress = DEVICE_PROGRESS_INITIALIZER;
-	progress.maximum = REEFNET_SENSUSULTRA_MEMORY_USER_SIZE + 2;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	progress.maximum = SZ_USER + 2;
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Wake-up the device and send the instruction code.
-	device_status_t rc = reefnet_sensusultra_send (device, 0xB430);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = reefnet_sensusultra_send (device, 0xB430);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	// Send the data to the device.
-	for (unsigned int i = 0; i < REEFNET_SENSUSULTRA_MEMORY_USER_SIZE; ++i) {
+	for (unsigned int i = 0; i < SZ_USER; ++i) {
 		rc = reefnet_sensusultra_send_uchar (device, data[i]);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		// Update and emit a progress event.
 		progress.current += 1;
-		device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 	}
 
 	// Send the checksum to the device.
-	unsigned short crc = checksum_crc_ccitt_uint16 (data, REEFNET_SENSUSULTRA_MEMORY_USER_SIZE);
+	unsigned short crc = checksum_crc_ccitt_uint16 (data, SZ_USER);
 	rc = reefnet_sensusultra_send_ushort (device, crc);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	// Update and emit a progress event.
 	progress.current += 2;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_device_write_parameter (device_t *abstract, reefnet_sensusultra_parameter_t parameter, unsigned int value)
+dc_status_t
+reefnet_sensusultra_device_write_parameter (dc_device_t *abstract, reefnet_sensusultra_parameter_t parameter, unsigned int value)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Set the instruction code and validate the new value.
 	unsigned short code = 0;
@@ -563,75 +550,75 @@ reefnet_sensusultra_device_write_parameter (device_t *abstract, reefnet_sensusul
 	case REEFNET_SENSUSULTRA_PARAMETER_INTERVAL:
 		code = 0xB410;
 		if (value < 1 || value > 65535)
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_INVALIDARGS;
 		break;
 	case REEFNET_SENSUSULTRA_PARAMETER_THRESHOLD:
 		code = 0xB411;
 		if (value < 1 || value > 65535)
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_INVALIDARGS;
 		break;
 	case REEFNET_SENSUSULTRA_PARAMETER_ENDCOUNT:
 		code = 0xB412;
 		if (value < 1 || value > 65535)
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_INVALIDARGS;
 		break;
 	case REEFNET_SENSUSULTRA_PARAMETER_AVERAGING:
 		code = 0xB413;
 		if (value != 1 && value != 2 && value != 4)
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_INVALIDARGS;
 		break;
 	default:
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 	}
 
 	// Wake-up the device and send the instruction code.
-	device_status_t rc = reefnet_sensusultra_send (device, code);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = reefnet_sensusultra_send (device, code);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	// Send the new value to the device.
 	rc = reefnet_sensusultra_send_ushort (device, value);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_device_sense (device_t *abstract, unsigned char *data, unsigned int size)
+dc_status_t
+reefnet_sensusultra_device_sense (dc_device_t *abstract, unsigned char *data, unsigned int size)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
-	if (size < REEFNET_SENSUSULTRA_SENSE_SIZE) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+	if (size < SZ_SENSE) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_INVALIDARGS;
 	}
 
 	// Wake-up the device and send the instruction code.
-	device_status_t rc = reefnet_sensusultra_send (device, 0xB440);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = reefnet_sensusultra_send (device, 0xB440);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	// Receive the packet.
-	unsigned char package[REEFNET_SENSUSULTRA_SENSE_SIZE + 2] = {0};
+	unsigned char package[SZ_SENSE + 2] = {0};
 	rc = reefnet_sensusultra_packet (device, package, sizeof (package), 0);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	memcpy (data, package, REEFNET_SENSUSULTRA_SENSE_SIZE);
+	memcpy (data, package, SZ_SENSE);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
+static dc_status_t
 reefnet_sensusultra_parse (reefnet_sensusultra_device_t *device,
 	const unsigned char data[], unsigned int *premaining, unsigned int *pprevious,
-	int *aborted, dive_callback_t callback, void *userdata)
+	int *aborted, dc_dive_callback_t callback, void *userdata)
 {
 	const unsigned char header[4] = {0x00, 0x00, 0x00, 0x00};
 	const unsigned char footer[4] = {0xFF, 0xFF, 0xFF, 0xFF};
@@ -669,13 +656,13 @@ reefnet_sensusultra_parse (reefnet_sensusultra_device_t *device,
 			if (device && timestamp <= device->timestamp) {
 				if (aborted)
 					*aborted = 1;
-				return DEVICE_STATUS_SUCCESS;
+				return DC_STATUS_SUCCESS;
 			}
 
 			if (callback && !callback (current, previous - current, current + 4, 4, userdata)) {
 				if (aborted)
 					*aborted = 1;
-				return DEVICE_STATUS_SUCCESS;
+				return DC_STATUS_SUCCESS;
 			}
 		}
 
@@ -694,32 +681,32 @@ reefnet_sensusultra_parse (reefnet_sensusultra_device_t *device,
 	if (aborted)
 		*aborted = 0;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-reefnet_sensusultra_device_foreach (device_t *abstract, dive_callback_t callback, void *userdata)
+static dc_status_t
+reefnet_sensusultra_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t*) abstract;
 
 	if (! device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
-	dc_buffer_t *buffer = dc_buffer_new (REEFNET_SENSUSULTRA_MEMORY_DATA_SIZE);
+	dc_buffer_t *buffer = dc_buffer_new (SZ_MEMORY);
 	if (buffer == NULL) {
-		WARNING ("Memory allocation error.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (abstract->context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Enable progress notifications.
-	device_progress_t progress = DEVICE_PROGRESS_INITIALIZER;
-	progress.maximum = REEFNET_SENSUSULTRA_MEMORY_DATA_SIZE;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	progress.maximum = SZ_MEMORY;
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Wake-up the device and send the instruction code.
-	device_status_t rc = reefnet_sensusultra_send (device, 0xB421);
-	if (rc != DEVICE_STATUS_SUCCESS) {
+	dc_status_t rc = reefnet_sensusultra_send (device, 0xB421);
+	if (rc != DC_STATUS_SUCCESS) {
 		dc_buffer_free (buffer);
 		return rc;
 	}
@@ -730,38 +717,38 @@ reefnet_sensusultra_device_foreach (device_t *abstract, dive_callback_t callback
 
 	unsigned int nbytes = 0;
 	unsigned int npages = 0;
-	while (nbytes < REEFNET_SENSUSULTRA_MEMORY_DATA_SIZE) {
+	while (nbytes < SZ_MEMORY) {
 		// Receive the packet.
-		unsigned char packet[REEFNET_SENSUSULTRA_PACKET_SIZE + 4] = {0};
+		unsigned char packet[SZ_PACKET + 4] = {0};
 		rc = reefnet_sensusultra_page (device, packet, sizeof (packet), npages);
-		if (rc != DEVICE_STATUS_SUCCESS) {
+		if (rc != DC_STATUS_SUCCESS) {
 			dc_buffer_free (buffer);
 			return rc;
 		}
 
 		// Update and emit a progress event.
-		progress.current += REEFNET_SENSUSULTRA_PACKET_SIZE;
-		device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+		progress.current += SZ_PACKET;
+		device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 		// Abort the transfer if the page contains no useful data.
-		if (array_isequal (packet + 2, REEFNET_SENSUSULTRA_PACKET_SIZE, 0xFF) && nbytes != 0)
+		if (array_isequal (packet + 2, SZ_PACKET, 0xFF) && nbytes != 0)
 			break;
 
 		// Prepend the packet to the buffer.
-		if (!dc_buffer_prepend (buffer, packet + 2, REEFNET_SENSUSULTRA_PACKET_SIZE)) {
-			WARNING ("Insufficient buffer space available.");
-			return DEVICE_STATUS_MEMORY;
+		if (!dc_buffer_prepend (buffer, packet + 2, SZ_PACKET)) {
+			ERROR (abstract->context, "Insufficient buffer space available.");
+			return DC_STATUS_NOMEMORY;
 		}
 
 		// Update the parser state.
-		remaining += REEFNET_SENSUSULTRA_PACKET_SIZE;
-		previous += REEFNET_SENSUSULTRA_PACKET_SIZE;
+		remaining += SZ_PACKET;
+		previous += SZ_PACKET;
 
 		// Parse the page data.
 		int aborted = 0;
 		rc = reefnet_sensusultra_parse (device, dc_buffer_get_data (buffer),
 			&remaining, &previous, &aborted, callback, userdata);
-		if (rc != DEVICE_STATUS_SUCCESS) {
+		if (rc != DC_STATUS_SUCCESS) {
 			dc_buffer_free (buffer);
 			return rc;
 		}
@@ -770,28 +757,28 @@ reefnet_sensusultra_device_foreach (device_t *abstract, dive_callback_t callback
 
 		// Accept the packet.
 		rc = reefnet_sensusultra_send_uchar (device, ACCEPT);
-		if (rc != DEVICE_STATUS_SUCCESS) {
+		if (rc != DC_STATUS_SUCCESS) {
 			dc_buffer_free (buffer);
 			return rc;
 		}
 
-		nbytes += REEFNET_SENSUSULTRA_PACKET_SIZE;
+		nbytes += SZ_PACKET;
 		npages++;
 	}
 
 	dc_buffer_free (buffer);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-reefnet_sensusultra_extract_dives (device_t *abstract, const unsigned char data[], unsigned int size, dive_callback_t callback, void *userdata)
+dc_status_t
+reefnet_sensusultra_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	reefnet_sensusultra_device_t *device = (reefnet_sensusultra_device_t *) abstract;
 
 	if (abstract && !device_is_reefnet_sensusultra (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	unsigned int remaining = size;
 	unsigned int previous = size;
