@@ -23,8 +23,8 @@
 #include <string.h> // memcmp, memcpy
 #include <assert.h> // assert
 
+#include "context-private.h"
 #include "suunto_common2.h"
-#include "utils.h"
 #include "ringbuffer.h"
 #include "checksum.h"
 #include "array.h"
@@ -35,33 +35,32 @@
 #define SZ_PACKET     0x78
 #define SZ_MINIMUM    8
 
-#define FP_OFFSET     0x15
-
 #define RB_PROFILE_DISTANCE(l,a,b,m)  ringbuffer_distance (a, b, m, l->rb_profile_begin, l->rb_profile_end)
 
 #define BACKEND(abstract)	((suunto_common2_device_backend_t *) abstract->backend)
 
 void
-suunto_common2_device_init (suunto_common2_device_t *device, const suunto_common2_device_backend_t *backend)
+suunto_common2_device_init (suunto_common2_device_t *device, dc_context_t *context, const suunto_common2_device_backend_t *backend)
 {
 	assert (device != NULL);
 
 	// Initialize the base class.
-	device_init (&device->base, &backend->base);
+	device_init (&device->base, context, &backend->base);
 
 	// Set the default values.
 	device->layout = NULL;
+	memset (device->version, 0, sizeof (device->version));
 	memset (device->fingerprint, 0, sizeof (device->fingerprint));
 }
 
 
-static device_status_t
-suunto_common2_transfer (device_t *abstract, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int size)
+static dc_status_t
+suunto_common2_transfer (dc_device_t *abstract, const unsigned char command[], unsigned int csize, unsigned char answer[], unsigned int asize, unsigned int size)
 {
 	assert (asize >= size + 4);
 
 	if (BACKEND (abstract)->packet == NULL)
-		return DEVICE_STATUS_UNSUPPORTED;
+		return DC_STATUS_UNSUPPORTED;
 
 	// Occasionally, the dive computer does not respond to a command.
 	// In that case we retry the command a number of times before
@@ -69,11 +68,11 @@ suunto_common2_transfer (device_t *abstract, const unsigned char command[], unsi
 	// again during one of the retries.
 
 	unsigned int nretries = 0;
-	device_status_t rc = DEVICE_STATUS_SUCCESS;
-	while ((rc = BACKEND (abstract)->packet (abstract, command, csize, answer, asize, size)) != DEVICE_STATUS_SUCCESS) {
+	dc_status_t rc = DC_STATUS_SUCCESS;
+	while ((rc = BACKEND (abstract)->packet (abstract, command, csize, answer, asize, size)) != DC_STATUS_SUCCESS) {
 		// Automatically discard a corrupted packet,
 		// and request a new one.
-		if (rc != DEVICE_STATUS_TIMEOUT && rc != DEVICE_STATUS_PROTOCOL)
+		if (rc != DC_STATUS_TIMEOUT && rc != DC_STATUS_PROTOCOL)
 			return rc;
 
 		// Abort if the maximum number of retries is reached.
@@ -85,62 +84,59 @@ suunto_common2_transfer (device_t *abstract, const unsigned char command[], unsi
 }
 
 
-device_status_t
-suunto_common2_device_set_fingerprint (device_t *abstract, const unsigned char data[], unsigned int size)
+dc_status_t
+suunto_common2_device_set_fingerprint (dc_device_t *abstract, const unsigned char data[], unsigned int size)
 {
 	suunto_common2_device_t *device = (suunto_common2_device_t*) abstract;
 
 	if (size && size != sizeof (device->fingerprint))
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	if (size)
 		memcpy (device->fingerprint, data, sizeof (device->fingerprint));
 	else
 		memset (device->fingerprint, 0, sizeof (device->fingerprint));
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_common2_device_version (device_t *abstract, unsigned char data[], unsigned int size)
+dc_status_t
+suunto_common2_device_version (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
 	if (size < SZ_VERSION) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_INVALIDARGS;
 	}
 
 	unsigned char answer[SZ_VERSION + 4] = {0};
 	unsigned char command[4] = {0x0F, 0x00, 0x00, 0x0F};
-	device_status_t rc = suunto_common2_transfer (abstract, command, sizeof (command), answer, sizeof (answer), 4);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = suunto_common2_transfer (abstract, command, sizeof (command), answer, sizeof (answer), 4);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
 	memcpy (data, answer + 3, SZ_VERSION);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_common2_device_reset_maxdepth (device_t *abstract)
+dc_status_t
+suunto_common2_device_reset_maxdepth (dc_device_t *abstract)
 {
 	unsigned char answer[4] = {0};
 	unsigned char command[4] = {0x20, 0x00, 0x00, 0x20};
-	device_status_t rc = suunto_common2_transfer (abstract, command, sizeof (command), answer, sizeof (answer), 0);
-	if (rc != DEVICE_STATUS_SUCCESS)
+	dc_status_t rc = suunto_common2_transfer (abstract, command, sizeof (command), answer, sizeof (answer), 0);
+	if (rc != DC_STATUS_SUCCESS)
 		return rc;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_common2_device_read (device_t *abstract, unsigned int address, unsigned char data[], unsigned int size)
+dc_status_t
+suunto_common2_device_read (dc_device_t *abstract, unsigned int address, unsigned char data[], unsigned int size)
 {
-	// The data transmission is split in packages
-	// of maximum $SZ_PACKET bytes.
-
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
 		// Calculate the package size.
@@ -156,8 +152,8 @@ suunto_common2_device_read (device_t *abstract, unsigned int address, unsigned c
 				len, // count
 				0};  // CRC
 		command[6] = checksum_xor_uint8 (command, 6, 0x00);
-		device_status_t rc = suunto_common2_transfer (abstract, command, sizeof (command), answer, len + 7, len);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		dc_status_t rc = suunto_common2_transfer (abstract, command, sizeof (command), answer, len + 7, len);
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		memcpy (data, answer + 6, len);
@@ -167,16 +163,13 @@ suunto_common2_device_read (device_t *abstract, unsigned int address, unsigned c
 		data += len;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_common2_device_write (device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size)
+dc_status_t
+suunto_common2_device_write (dc_device_t *abstract, unsigned int address, const unsigned char data[], unsigned int size)
 {
-	// The data transmission is split in packages
-	// of maximum $SZ_PACKET bytes.
-
 	unsigned int nbytes = 0;
 	while (nbytes < size) {
 		// Calculate the package size.
@@ -193,8 +186,8 @@ suunto_common2_device_write (device_t *abstract, unsigned int address, const uns
 				0};  // data + CRC
 		memcpy (command + 6, data, len);
 		command[len + 6] = checksum_xor_uint8 (command, len + 6, 0x00);
-		device_status_t rc = suunto_common2_transfer (abstract, command, len + 7, answer, sizeof (answer), 0);
-		if (rc != DEVICE_STATUS_SUCCESS)
+		dc_status_t rc = suunto_common2_transfer (abstract, command, len + 7, answer, sizeof (answer), 0);
+		if (rc != DC_STATUS_SUCCESS)
 			return rc;
 
 		nbytes += len;
@@ -202,12 +195,12 @@ suunto_common2_device_write (device_t *abstract, unsigned int address, const uns
 		data += len;
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_common2_device_dump (device_t *abstract, dc_buffer_t *buffer)
+dc_status_t
+suunto_common2_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	suunto_common2_device_t *device = (suunto_common2_device_t *) abstract;
 
@@ -217,17 +210,23 @@ suunto_common2_device_dump (device_t *abstract, dc_buffer_t *buffer)
 	// Erase the current contents of the buffer and
 	// allocate the required amount of memory.
 	if (!dc_buffer_clear (buffer) || !dc_buffer_resize (buffer, device->layout->memsize)) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_NOMEMORY;
 	}
 
+	// Emit a vendor event.
+	dc_event_vendor_t vendor;
+	vendor.data = device->version;
+	vendor.size = sizeof (device->version);
+	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
+
 	return device_dump_read (abstract, dc_buffer_get_data (buffer),
 		dc_buffer_get_size (buffer), SZ_PACKET);
 }
 
 
-device_status_t
-suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, void *userdata)
+dc_status_t
+suunto_common2_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
 	suunto_common2_device_t *device = (suunto_common2_device_t*) abstract;
 
@@ -237,50 +236,48 @@ suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, voi
 	const suunto_common2_layout_t *layout = device->layout;
 
 	// Error status for delayed errors.
-	device_status_t status = DEVICE_STATUS_SUCCESS;
+	dc_status_t status = DC_STATUS_SUCCESS;
 
 	// Enable progress notifications.
-	device_progress_t progress = DEVICE_PROGRESS_INITIALIZER;
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
 	progress.maximum = layout->rb_profile_end - layout->rb_profile_begin +
-		8 + SZ_VERSION + (SZ_MINIMUM > 4 ? SZ_MINIMUM : 4);
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+		8 + (SZ_MINIMUM > 4 ? SZ_MINIMUM : 4);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
-	// Read the version info.
-	unsigned char version[SZ_VERSION] = {0};
-	device_status_t rc = suunto_common2_device_version (abstract, version, sizeof (version));
-	if (rc != DEVICE_STATUS_SUCCESS) {
-		WARNING ("Cannot read memory header.");
-		return rc;
-	}
-
-	// Update and emit a progress event.
-	progress.current += sizeof (version);
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	// Emit a vendor event.
+	dc_event_vendor_t vendor;
+	vendor.data = device->version;
+	vendor.size = sizeof (device->version);
+	device_event_emit (abstract, DC_EVENT_VENDOR, &vendor);
 
 	// Read the serial number.
 	unsigned char serial[SZ_MINIMUM > 4 ? SZ_MINIMUM : 4] = {0};
-	rc = suunto_common2_device_read (abstract, layout->serial, serial, sizeof (serial));
-	if (rc != DEVICE_STATUS_SUCCESS) {
-		WARNING ("Cannot read memory header.");
+	dc_status_t rc = suunto_common2_device_read (abstract, layout->serial, serial, sizeof (serial));
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the memory header.");
 		return rc;
 	}
 
 	// Update and emit a progress event.
 	progress.current += sizeof (serial);
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Emit a device info event.
-	device_devinfo_t devinfo;
-	devinfo.model = version[0];
-	devinfo.firmware = array_uint24_be (version + 1);
-	devinfo.serial = array_uint32_be (serial);
-	device_event_emit (abstract, DEVICE_EVENT_DEVINFO, &devinfo);
+	dc_event_devinfo_t devinfo;
+	devinfo.model = device->version[0];
+	devinfo.firmware = array_uint24_be (device->version + 1);
+	devinfo.serial = 0;
+	for (unsigned int i = 0; i < 4; ++i) {
+		devinfo.serial *= 100;
+		devinfo.serial += serial[i];
+	}
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	// Read the header bytes.
 	unsigned char header[8] = {0};
 	rc = suunto_common2_device_read (abstract, 0x0190, header, sizeof (header));
-	if (rc != DEVICE_STATUS_SUCCESS) {
-		WARNING ("Cannot read memory header.");
+	if (rc != DC_STATUS_SUCCESS) {
+		ERROR (abstract->context, "Failed to read the memory header.");
 		return rc;
 	}
 
@@ -296,16 +293,16 @@ suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, voi
 		begin < layout->rb_profile_begin ||
 		begin >= layout->rb_profile_end)
 	{
-		WARNING("Invalid ringbuffer pointer detected!");
-		return DEVICE_STATUS_ERROR;
+		ERROR (abstract->context, "Invalid ringbuffer pointer detected.");
+		return DC_STATUS_DATAFORMAT;
 	}
 
 	// Memory buffer to store all the dives.
 
 	unsigned char *data = (unsigned char *) malloc (layout->rb_profile_end - layout->rb_profile_begin + SZ_MINIMUM);
 	if (data == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (abstract->context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Calculate the total amount of bytes.
@@ -316,7 +313,7 @@ suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, voi
 
 	progress.maximum -= (layout->rb_profile_end - layout->rb_profile_begin) - remaining;
 	progress.current += sizeof (header);
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// To reduce the number of read operations, we always try to read
 	// packages with the largest possible size. As a consequence, the
@@ -338,9 +335,9 @@ suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, voi
 		unsigned int size = RB_PROFILE_DISTANCE (layout, current, previous, 1);
 
 		if (size < 4 || size > remaining) {
-			WARNING ("Unexpected profile size.");
+			ERROR (abstract->context, "Unexpected profile size.");
 			free (data);
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_DATAFORMAT;
 		}
 
 		unsigned int nbytes = available;
@@ -374,15 +371,15 @@ suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, voi
 
 			// Read the package.
 			rc = suunto_common2_device_read (abstract, address - extra, data + offset - extra, len + extra);
-			if (rc != DEVICE_STATUS_SUCCESS) {
-				WARNING ("Cannot read memory.");
+			if (rc != DC_STATUS_SUCCESS) {
+				ERROR (abstract->context, "Failed to read the memory.");
 				free (data);
 				return rc;
 			}
 
 			// Update and emit a progress event.
 			progress.current += len;
-			device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+			device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 			// Next package.
 			nbytes += len;
@@ -403,33 +400,30 @@ suunto_common2_device_foreach (device_t *abstract, dive_callback_t callback, voi
 			next < layout->rb_profile_begin ||
 			next >= layout->rb_profile_end)
 		{
-			WARNING("Invalid ringbuffer pointer detected!");
+			ERROR (abstract->context, "Invalid ringbuffer pointer detected.");
 			free (data);
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_DATAFORMAT;
 		}
 		if (next != previous && next != current) {
-			WARNING ("Profiles are not continuous.");
+			ERROR (abstract->context, "Profiles are not continuous.");
 			free (data);
-			return DEVICE_STATUS_ERROR;
+			return DC_STATUS_DATAFORMAT;
 		}
 
 		if (next != current) {
-			unsigned int fp_offset = FP_OFFSET;
-			if (devinfo.model == 0x15)
-				fp_offset += 6; // HelO2
-
+			unsigned int fp_offset = layout->fingerprint + 4;
 			if (memcmp (p + fp_offset, device->fingerprint, sizeof (device->fingerprint)) == 0) {
 				free (data);
-				return DEVICE_STATUS_SUCCESS;
+				return DC_STATUS_SUCCESS;
 			}
 
 			if (callback && !callback (p + 4, size - 4, p + fp_offset, sizeof (device->fingerprint), userdata)) {
 				free (data);
-				return DEVICE_STATUS_SUCCESS;
+				return DC_STATUS_SUCCESS;
 			}
 		} else {
-			WARNING ("Skipping incomplete dive.");
-			status = DEVICE_STATUS_ERROR;
+			ERROR (abstract->context, "Skipping incomplete dive.");
+			status = DC_STATUS_DATAFORMAT;
 		}
 
 		// Next dive.
