@@ -22,32 +22,34 @@
 #include <string.h> // memcmp, memcpy
 #include <stdlib.h> // malloc, free
 
+#include <libdivecomputer/suunto_eon.h>
+
+#include "context-private.h"
 #include "device-private.h"
-#include "suunto_eon.h"
 #include "suunto_common.h"
 #include "serial.h"
 #include "checksum.h"
-#include "utils.h"
 #include "array.h"
 
 #define EXITCODE(rc) \
 ( \
-	rc == -1 ? DEVICE_STATUS_IO : DEVICE_STATUS_TIMEOUT \
+	rc == -1 ? DC_STATUS_IO : DC_STATUS_TIMEOUT \
 )
 
+#define SZ_MEMORY 0x900
+
 typedef struct suunto_eon_device_t {
 	suunto_common_device_t base;
 	serial_t *port;
 } suunto_eon_device_t;
 
-static device_status_t suunto_eon_device_dump (device_t *abstract, dc_buffer_t *buffer);
-static device_status_t suunto_eon_device_foreach (device_t *abstract, dive_callback_t callback, void *userdata);
-static device_status_t suunto_eon_device_close (device_t *abstract);
+static dc_status_t suunto_eon_device_dump (dc_device_t *abstract, dc_buffer_t *buffer);
+static dc_status_t suunto_eon_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata);
+static dc_status_t suunto_eon_device_close (dc_device_t *abstract);
 
 static const device_backend_t suunto_eon_device_backend = {
-	DEVICE_TYPE_SUUNTO_EON,
+	DC_FAMILY_SUUNTO_EON,
 	suunto_common_device_set_fingerprint, /* set_fingerprint */
-	NULL, /* version */
 	NULL, /* read */
 	NULL, /* write */
 	suunto_eon_device_dump, /* dump */
@@ -58,14 +60,14 @@ static const device_backend_t suunto_eon_device_backend = {
 static const suunto_common_layout_t suunto_eon_layout = {
 	0, /* eop */
 	0x100, /* rb_profile_begin */
-	SUUNTO_EON_MEMORY_SIZE, /* rb_profile_end */
+	SZ_MEMORY, /* rb_profile_end */
 	6, /* fp_offset */
 	3 /* peek */
 };
 
 
 static int
-device_is_suunto_eon (device_t *abstract)
+device_is_suunto_eon (dc_device_t *abstract)
 {
 	if (abstract == NULL)
 		return 0;
@@ -74,159 +76,163 @@ device_is_suunto_eon (device_t *abstract)
 }
 
 
-device_status_t
-suunto_eon_device_open (device_t **out, const char* name)
+dc_status_t
+suunto_eon_device_open (dc_device_t **out, dc_context_t *context, const char *name)
 {
 	if (out == NULL)
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
 	suunto_eon_device_t *device = (suunto_eon_device_t *) malloc (sizeof (suunto_eon_device_t));
 	if (device == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return DEVICE_STATUS_MEMORY;
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	suunto_common_device_init (&device->base, &suunto_eon_device_backend);
+	suunto_common_device_init (&device->base, context, &suunto_eon_device_backend);
 
 	// Set the default values.
 	device->port = NULL;
 
 	// Open the device.
-	int rc = serial_open (&device->port, name);
+	int rc = serial_open (&device->port, context, name);
 	if (rc == -1) {
-		WARNING ("Failed to open the serial port.");
+		ERROR (context, "Failed to open the serial port.");
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Set the serial communication protocol (1200 8N2).
 	rc = serial_configure (device->port, 1200, 8, SERIAL_PARITY_NONE, 2, SERIAL_FLOWCONTROL_NONE);
 	if (rc == -1) {
-		WARNING ("Failed to set the terminal attributes.");
+		ERROR (context, "Failed to set the terminal attributes.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Set the timeout for receiving data (1000ms).
 	if (serial_set_timeout (device->port, -1) == -1) {
-		WARNING ("Failed to set the timeout.");
+		ERROR (context, "Failed to set the timeout.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Clear the RTS line.
 	if (serial_set_rts (device->port, 0)) {
-		WARNING ("Failed to set the DTR/RTS line.");
+		ERROR (context, "Failed to set the DTR/RTS line.");
 		serial_close (device->port);
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
-	*out = (device_t*) device;
+	*out = (dc_device_t*) device;
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-suunto_eon_device_close (device_t *abstract)
+static dc_status_t
+suunto_eon_device_close (dc_device_t *abstract)
 {
 	suunto_eon_device_t *device = (suunto_eon_device_t*) abstract;
 
 	if (! device_is_suunto_eon (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Close the device.
 	if (serial_close (device->port) == -1) {
 		free (device);
-		return DEVICE_STATUS_IO;
+		return DC_STATUS_IO;
 	}
 
 	// Free memory.	
 	free (device);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-suunto_eon_device_dump (device_t *abstract, dc_buffer_t *buffer)
+static dc_status_t
+suunto_eon_device_dump (dc_device_t *abstract, dc_buffer_t *buffer)
 {
 	suunto_eon_device_t *device = (suunto_eon_device_t*) abstract;
 
 	if (! device_is_suunto_eon (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Erase the current contents of the buffer and
 	// pre-allocate the required amount of memory.
-	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, SUUNTO_EON_MEMORY_SIZE)) {
-		WARNING ("Insufficient buffer space available.");
-		return DEVICE_STATUS_MEMORY;
+	if (!dc_buffer_clear (buffer) || !dc_buffer_reserve (buffer, SZ_MEMORY)) {
+		ERROR (abstract->context, "Insufficient buffer space available.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Enable progress notifications.
-	device_progress_t progress = DEVICE_PROGRESS_INITIALIZER;
-	progress.maximum = SUUNTO_EON_MEMORY_SIZE + 1;
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	dc_event_progress_t progress = EVENT_PROGRESS_INITIALIZER;
+	progress.maximum = SZ_MEMORY + 1;
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Send the command.
 	unsigned char command[1] = {'P'};
 	int rc = serial_write (device->port, command, sizeof (command));
 	if (rc != sizeof (command)) {
-		WARNING ("Failed to send the command.");
+		ERROR (abstract->context, "Failed to send the command.");
 		return EXITCODE (rc);
 	}
 
 	// Receive the answer.
-	unsigned char answer[SUUNTO_EON_MEMORY_SIZE + 1] = {0};
+	unsigned char answer[SZ_MEMORY + 1] = {0};
 	rc = serial_read (device->port, answer, sizeof (answer));
 	if (rc != sizeof (answer)) {
-		WARNING ("Failed to receive the answer.");
+		ERROR (abstract->context, "Failed to receive the answer.");
 		return EXITCODE (rc);
 	}
 
 	// Update and emit a progress event.
 	progress.current += sizeof (answer);
-	device_event_emit (abstract, DEVICE_EVENT_PROGRESS, &progress);
+	device_event_emit (abstract, DC_EVENT_PROGRESS, &progress);
 
 	// Verify the checksum of the package.
 	unsigned char crc = answer[sizeof (answer) - 1];
 	unsigned char ccrc = checksum_add_uint8 (answer, sizeof (answer) - 1, 0x00);
 	if (crc != ccrc) {
-		WARNING ("Unexpected answer CRC.");
-		return DEVICE_STATUS_PROTOCOL;
+		ERROR (abstract->context, "Unexpected answer checksum.");
+		return DC_STATUS_PROTOCOL;
 	}
 
-	dc_buffer_append (buffer, answer, SUUNTO_EON_MEMORY_SIZE);
+	dc_buffer_append (buffer, answer, SZ_MEMORY);
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static device_status_t
-suunto_eon_device_foreach (device_t *abstract, dive_callback_t callback, void *userdata)
+static dc_status_t
+suunto_eon_device_foreach (dc_device_t *abstract, dc_dive_callback_t callback, void *userdata)
 {
-	dc_buffer_t *buffer = dc_buffer_new (SUUNTO_EON_MEMORY_SIZE);
+	dc_buffer_t *buffer = dc_buffer_new (SZ_MEMORY);
 	if (buffer == NULL)
-		return DEVICE_STATUS_MEMORY;
+		return DC_STATUS_NOMEMORY;
 
-	device_status_t rc = suunto_eon_device_dump (abstract, buffer);
-	if (rc != DEVICE_STATUS_SUCCESS) {
+	dc_status_t rc = suunto_eon_device_dump (abstract, buffer);
+	if (rc != DC_STATUS_SUCCESS) {
 		dc_buffer_free (buffer);
 		return rc;
 	}
 
 	// Emit a device info event.
 	unsigned char *data = dc_buffer_get_data (buffer);
-	device_devinfo_t devinfo;
+	dc_event_devinfo_t devinfo;
 	devinfo.model = 0;
 	devinfo.firmware = 0;
-	devinfo.serial = array_uint24_be (data + 244);
-	device_event_emit (abstract, DEVICE_EVENT_DEVINFO, &devinfo);
+	devinfo.serial = 0;
+	for (unsigned int i = 0; i < 3; ++i) {
+		devinfo.serial *= 100;
+		devinfo.serial += bcd2dec (data[244 + i]);
+	}
+	device_event_emit (abstract, DC_EVENT_DEVINFO, &devinfo);
 
 	rc = suunto_eon_extract_dives (abstract,
 		dc_buffer_get_data (buffer), dc_buffer_get_size (buffer), callback, userdata);
@@ -237,60 +243,60 @@ suunto_eon_device_foreach (device_t *abstract, dive_callback_t callback, void *u
 }
 
 
-device_status_t
-suunto_eon_device_write_name (device_t *abstract, unsigned char data[], unsigned int size)
+dc_status_t
+suunto_eon_device_write_name (dc_device_t *abstract, unsigned char data[], unsigned int size)
 {
 	suunto_eon_device_t *device = (suunto_eon_device_t*) abstract;
 
 	if (! device_is_suunto_eon (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	if (size > 20)
-		return DEVICE_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Send the command.
 	unsigned char command[21] = {'N'};
 	memcpy (command + 1, data, size);
 	int rc = serial_write (device->port, command, sizeof (command));
 	if (rc != sizeof (command)) {
-		WARNING ("Failed to send the command.");
+		ERROR (abstract->context, "Failed to send the command.");
 		return EXITCODE (rc);
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_eon_device_write_interval (device_t *abstract, unsigned char interval)
+dc_status_t
+suunto_eon_device_write_interval (dc_device_t *abstract, unsigned char interval)
 {
 	suunto_eon_device_t *device = (suunto_eon_device_t*) abstract;
 
 	if (! device_is_suunto_eon (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Send the command.
 	unsigned char command[2] = {'T', interval};
 	int rc = serial_write (device->port, command, sizeof (command));
 	if (rc != sizeof (command)) {
-		WARNING ("Failed to send the command.");
+		ERROR (abstract->context, "Failed to send the command.");
 		return EXITCODE (rc);
 	}
 
-	return DEVICE_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-device_status_t
-suunto_eon_extract_dives (device_t *abstract, const unsigned char data[], unsigned int size, dive_callback_t callback, void *userdata)
+dc_status_t
+suunto_eon_extract_dives (dc_device_t *abstract, const unsigned char data[], unsigned int size, dc_dive_callback_t callback, void *userdata)
 {
 	suunto_common_device_t *device = (suunto_common_device_t*) abstract;
 
 	if (abstract && !device_is_suunto_eon (abstract))
-		return DEVICE_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
-	if (size < SUUNTO_EON_MEMORY_SIZE)
-		return DEVICE_STATUS_ERROR;
+	if (size < SZ_MEMORY)
+		return DC_STATUS_DATAFORMAT;
 
 	return suunto_common_extract_dives (device, &suunto_eon_layout, data, callback, userdata);
 }
