@@ -21,28 +21,29 @@
 
 #include <stdlib.h>
 
-#include "suunto_solution.h"
+#include <libdivecomputer/suunto_solution.h>
+#include <libdivecomputer/units.h>
+
+#include "context-private.h"
 #include "parser-private.h"
-#include "units.h"
-#include "utils.h"
 
 typedef struct suunto_solution_parser_t suunto_solution_parser_t;
 
 struct suunto_solution_parser_t {
-	parser_t base;
+	dc_parser_t base;
 	// Cached fields.
 	unsigned int cached;
 	unsigned int divetime;
 	unsigned int maxdepth;
 };
 
-static parser_status_t suunto_solution_parser_set_data (parser_t *abstract, const unsigned char *data, unsigned int size);
-static parser_status_t suunto_solution_parser_get_field (parser_t *abstract, parser_field_type_t type, unsigned int flags, void *value);
-static parser_status_t suunto_solution_parser_samples_foreach (parser_t *abstract, sample_callback_t callback, void *userdata);
-static parser_status_t suunto_solution_parser_destroy (parser_t *abstract);
+static dc_status_t suunto_solution_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size);
+static dc_status_t suunto_solution_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value);
+static dc_status_t suunto_solution_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata);
+static dc_status_t suunto_solution_parser_destroy (dc_parser_t *abstract);
 
 static const parser_backend_t suunto_solution_parser_backend = {
-	PARSER_TYPE_SUUNTO_SOLUTION,
+	DC_FAMILY_SUUNTO_SOLUTION,
 	suunto_solution_parser_set_data, /* set_data */
 	NULL, /* datetime */
 	suunto_solution_parser_get_field, /* fields */
@@ -52,7 +53,7 @@ static const parser_backend_t suunto_solution_parser_backend = {
 
 
 static int
-parser_is_suunto_solution (parser_t *abstract)
+parser_is_suunto_solution (dc_parser_t *abstract)
 {
 	if (abstract == NULL)
 		return 0;
@@ -61,65 +62,65 @@ parser_is_suunto_solution (parser_t *abstract)
 }
 
 
-parser_status_t
-suunto_solution_parser_create (parser_t **out)
+dc_status_t
+suunto_solution_parser_create (dc_parser_t **out, dc_context_t *context)
 {
 	if (out == NULL)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_INVALIDARGS;
 
 	// Allocate memory.
 	suunto_solution_parser_t *parser = (suunto_solution_parser_t *) malloc (sizeof (suunto_solution_parser_t));
 	if (parser == NULL) {
-		WARNING ("Failed to allocate memory.");
-		return PARSER_STATUS_MEMORY;
+		ERROR (context, "Failed to allocate memory.");
+		return DC_STATUS_NOMEMORY;
 	}
 
 	// Initialize the base class.
-	parser_init (&parser->base, &suunto_solution_parser_backend);
+	parser_init (&parser->base, context, &suunto_solution_parser_backend);
 
 	// Set the default values.
 	parser->cached = 0;
 	parser->divetime = 0;
 	parser->maxdepth = 0;
 
-	*out = (parser_t*) parser;
+	*out = (dc_parser_t*) parser;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-suunto_solution_parser_destroy (parser_t *abstract)
+static dc_status_t
+suunto_solution_parser_destroy (dc_parser_t *abstract)
 {
 	if (! parser_is_suunto_solution (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Free memory.
 	free (abstract);
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-suunto_solution_parser_set_data (parser_t *abstract, const unsigned char *data, unsigned int size)
+static dc_status_t
+suunto_solution_parser_set_data (dc_parser_t *abstract, const unsigned char *data, unsigned int size)
 {
 	suunto_solution_parser_t *parser = (suunto_solution_parser_t *) abstract;
 
 	if (! parser_is_suunto_solution (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	// Reset the cache.
 	parser->cached = 0;
 	parser->divetime = 0;
 	parser->maxdepth = 0;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
 
-static parser_status_t
-suunto_solution_parser_get_field (parser_t *abstract, parser_field_type_t type, unsigned int flags, void *value)
+static dc_status_t
+suunto_solution_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
 {
 	suunto_solution_parser_t *parser = (suunto_solution_parser_t *) abstract;
 
@@ -127,7 +128,7 @@ suunto_solution_parser_get_field (parser_t *abstract, parser_field_type_t type,
 	unsigned int size = abstract->size;
 
 	if (size < 4)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	if (!parser->cached) {
 		unsigned int nsamples = 0;
@@ -139,7 +140,7 @@ suunto_solution_parser_get_field (parser_t *abstract, parser_field_type_t type,
 				depth += (signed char) value;
 				if (value == 0x7D || value == 0x83) {
 					if (offset + 1 > size)
-						return PARSER_STATUS_ERROR;
+						return DC_STATUS_DATAFORMAT;
 					depth += (signed char) data[offset++];
 				}
 				if (depth > maxdepth)
@@ -151,62 +152,62 @@ suunto_solution_parser_get_field (parser_t *abstract, parser_field_type_t type,
 		// Store the offset to the end marker.
 		unsigned int marker = offset;
 		if (marker + 1 >= size || data[marker] != 0x80)
-			return PARSER_STATUS_ERROR;
+			return DC_STATUS_DATAFORMAT;
 
 		parser->cached = 1;
 		parser->divetime = (nsamples * 3 + data[marker + 1]) * 60;
 		parser->maxdepth = maxdepth;
 	}
 
-	gasmix_t *gasmix = (gasmix_t *) value;
+	dc_gasmix_t *gasmix = (dc_gasmix_t *) value;
 
 	if (value) {
 		switch (type) {
-		case FIELD_TYPE_DIVETIME:
+		case DC_FIELD_DIVETIME:
 			*((unsigned int *) value) = parser->divetime;
 			break;
-		case FIELD_TYPE_MAXDEPTH:
+		case DC_FIELD_MAXDEPTH:
 			*((double *) value) = parser->maxdepth * FEET;
 			break;
-		case FIELD_TYPE_GASMIX_COUNT:
+		case DC_FIELD_GASMIX_COUNT:
 			*((unsigned int *) value) = 1;
 			break;
-		case FIELD_TYPE_GASMIX:
+		case DC_FIELD_GASMIX:
 			gasmix->helium = 0.0;
 			gasmix->oxygen = 0.21;
 			gasmix->nitrogen = 1.0 - gasmix->oxygen - gasmix->helium;
 			break;
 		default:
-			return PARSER_STATUS_UNSUPPORTED;
+			return DC_STATUS_UNSUPPORTED;
 		}
 	}
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
 
-static parser_status_t
-suunto_solution_parser_samples_foreach (parser_t *abstract, sample_callback_t callback, void *userdata)
+static dc_status_t
+suunto_solution_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t callback, void *userdata)
 {
 	if (! parser_is_suunto_solution (abstract))
-		return PARSER_STATUS_TYPE_MISMATCH;
+		return DC_STATUS_INVALIDARGS;
 
 	const unsigned char *data = abstract->data;
 	unsigned int size = abstract->size;
 
 	if (size < 4)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
 	unsigned int time = 0, depth = 0;
 
 	unsigned int offset = 3;
 	while (offset < size &&	data[offset] != 0x80) {
-		parser_sample_value_t sample = {0};
+		dc_sample_value_t sample = {0};
 		unsigned char value = data[offset++];
 		if (value < 0x7e || value > 0x82) {
 			// Time (minutes).
 			time += 3 * 60;
 			sample.time = time;
-			if (callback) callback (SAMPLE_TYPE_TIME, sample, userdata);
+			if (callback) callback (DC_SAMPLE_TIME, sample, userdata);
 
 			// Depth (ft).
 			depth += (signed char) value;
@@ -215,11 +216,11 @@ suunto_solution_parser_samples_foreach (parser_t *abstract, sample_callback_t ca
 				// or ascent greater than 124 feet. The remaining part of
 				// the total delta value is stored in the next byte.
 				if (offset + 1 > size)
-					return PARSER_STATUS_ERROR;
+					return DC_STATUS_DATAFORMAT;
 				depth += (signed char) data[offset++];
 			}
 			sample.depth = depth * FEET;
-			if (callback) callback (SAMPLE_TYPE_DEPTH, sample, userdata);
+			if (callback) callback (DC_SAMPLE_DEPTH, sample, userdata);
 		} else {
 			// Event.
 			sample.event.time = 0;
@@ -236,16 +237,16 @@ suunto_solution_parser_samples_foreach (parser_t *abstract, sample_callback_t ca
 				sample.event.type = SAMPLE_EVENT_ASCENT;
 				break;
 			default: // Unknown
-				WARNING ("Unknown event");
+				WARNING (abstract->context, "Unknown event");
 				break;
 			}
 
-			if (callback) callback (SAMPLE_TYPE_EVENT, sample, userdata);
+			if (callback) callback (DC_SAMPLE_EVENT, sample, userdata);
 		}
 	}
 
 	if (data[offset] != 0x80)
-		return PARSER_STATUS_ERROR;
+		return DC_STATUS_DATAFORMAT;
 
-	return PARSER_STATUS_SUCCESS;
+	return DC_STATUS_SUCCESS;
 }
